/*CRESTComponent.h
* Hi - I'm a CREST crystal Component module.  I contain a pointer to the physical node in the TGeoManager hierarchy that corresponds to where I live.  I can also be derived to contain component specific data (i.e. if I'm a tube, I have adc spectra, etc.)
* I am also a TEveGeoShape - the geometry data can also be stored here. And I can be added to the geometry scene (or any other scene) in a hierarchical manner.
*/

#ifndef CRESTCOMPONENT_H
#define CRESTCOMPONENT_H

#include "Riostream.h"
#include "TROOT.h"
#include "TEveGeoShape.h"

class UC_Event;
class TGeoHMatrix;
class TGeoShape;

//_______________________________________________________________________________
class CRESTComponent : public TEveGeoShape //inherit from TEveGeoShape -> place individually into a TEveElement list -> now gEve (The TEveManager) knows WHO (my special data) and WHERE (TEveGeoShape data)i am!
{
	public:
		CRESTComponent(const char* name="Component", const char* title="Component", Int_t id=0, TGeoHMatrix* trans=0, TGeoShape* shape=0, Bool_t is_assembly=kFALSE); 
		Int_t fCRESTId; //global ID of this detector element. Not the same as fTubeNum of UC_Event fame. Also less useful.
		Int_t GetId() const;
		Bool_t fIsAssembly;
		Float_t fXLoc; //even assemblies have locations, so let's put this info here.
		Float_t fYLoc;
		Float_t fZLoc;
		//TEveElement* ForwardEdit(); //this should return the element to be displayed in GUI editor on click.
		//might need to make a small GUI class to accomplish this? Or use default?
		//To implement this, make a editor class, e.g. for MyClass, make MyClassEditor, and the rest is AUTOMAGIC. THANK YOU ROOT.


		ClassDef(CRESTComponent, 5)   // Eve/CREST representation of a CREST sensitive detector component.
};

#endif 
