/*CRESTCrystal.h
* Hi - I'm a CREST Crystal module.  I have an X and a Y location. I store a vector of hits which is emptied every time we move from event to event. I live in a hierarchy of CRESTComponents owing to our common heritage as TEveGeoShape's.
*/

#ifndef CRESTCRYSTAL_H
#define CRESTCRYSTAL_H

#include "Riostream.h"
#include "TROOT.h"
#include "CRESTComponent.h"
#include "TH1F.h"

class UC_Event;

//_______________________________________________________________________________
class CRESTCrystal : public CRESTComponent 
{
	private:
		//Important Inherited Members - - - - - - - - - - - - - - - - - - - - - - - - - - 
		//Int_t fCRESTId; //global ID of this detector element. 
		//Int_t fEventMember; //True if member of the active event; false otherwise.  This must be controlled by the event loading process - the components have no internal way of figuring this out. Should not be set/unset directly but by the Bool_t ProcessEvent member function. Tubes should return 1 or 0; paddles can return 2, 1 or 0. 2 = a paddle pair.
		//End Important Inherited Members - - - - - - - - - - - - - - - - - - - - - - - - - - 
		//vector<CRESTHit> fHits; //yet to be designed CRESTHits class. Reset after each event.
	public:
		CRESTCrystal(const char* name = "crystal", const char* title = "crystal", Int_t id = -1, TGeoHMatrix* trans=0, TGeoShape* shape=0,Bool_t is_assembly=kTRUE); 
		//Important Non-Overridden Inherited Members - - - - - - - - - - - - - - - - - - - - - - - - - - 
		//Int_t GetId() const;
		//Bool_t fIsAssembly;
		//Float_t fZLoc; 
		//Float_t fYLoc;
		//Float_t fXLoc; 
		//End Important Inherited Members - - - - - - - - - - - - - - - - - - - - - - - - - - 

		//virtual Int_t ProcessEvent(UC_Event*);  //override this - special for cystal .
		TString GetHighlightTooltip(); //override this - print more info than just the title.

		Int_t fTubeNum; //corresponds to that in UC_Geometry (unlike fCRESTId, which doesn't.)
		Int_t fBus;
		Int_t fSedecim;
		Int_t fPin;
		TH1F	fADC0Hist;
		TH1F	fADC1Hist;
		TH1F* GetADCHist(Int_t type);

		ClassDef(CRESTCrystal, 4)   // Eve/CREST representation of a CREST detector component.
};

#endif 
