// Author: Bertrand Bellenot   22/08/02

/*************************************************************************
 * Copyright (C) 1995-2002, Bertrand Bellenot.                           *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see the LICENSE file.                         *
 *************************************************************************/

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// CRESTDetector                                                           //
// defines a simple detector class with one geometry, one material      //
// and the physical properties of the detector's material               //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef CRESTDETECTOR_H
#define CRESTDETECTOR_H

#include "constants.h"

#include "TObject.h"
#include "TObjArray.h"
#include "TGeoManager.h"
#include "TGeoMaterial.h"
#include "TGeoMedium.h"
#include "TGeoVolume.h"
#include "TGeoMatrix.h"

class CRESTDetector : public TObject {

private:
    Double_t        fMinX,fMinY,fMinZ; // Detector min. boundaries
    Double_t        fMaxX,fMaxY,fMaxZ; // Detector max. boundaries
    Double_t        fDimX,fDimY,fDimZ; // Detector dimensions

		TGeoVolume* 		fGVTop;

public:
    CRESTDetector();
    virtual ~CRESTDetector();
    void        Init();
    //Double_t    GetMinZ() { return fMinZ; }
    void        GetDimensions(Double_t *dimx, Double_t *dimy, Double_t *dimz)
                { *dimx = fDimX; *dimy = fDimY; *dimz = fDimZ; }
    TGeoManager *GetGeoManager() { return gGeoManager; }

    ClassDef(CRESTDetector,1)   // Detector structure
};

#endif // CRESTDETECTOR_H

