/*CRESTHit.h
* Hi - I'm a CREST crystal Hit module.  I contain a pointer to the physical node in the TGeoManager hierarchy that corresponds to where I live.  I can also be derived to contain component specific data (i.e. if I'm a tube, I have adc spectra, etc.)
* I am also a TEveGeoShape - the geometry data can also be stored here. And I can be added to the geometry scene (or any other scene) in a hierarchical manner.
*/

#ifndef CRESTHIT_H
#define CRESTHIT_H

#include <iostream>
//#include "TROOT.h"
//#include "TEveGeoShape.h"
#include "TObject.h"
//#include "UC_Event.h" //need to include since we'll use the info in a given array index of UC_Event to create ourselves

class CRESTComponent;
class UC_Event;
class TEveGeoShape;

//_______________________________________________________________________________
class CRESTHit : public TObject 
{
	public:
		CRESTHit(UC_Event* evt=0, Int_t idx=-1, CRESTComponent* detector=0);
		Int_t fHitIndex; //index in the UC_Event that spawned us
		Int_t fEventId; //id of the UC_Event that spawned us (starts at 0 in each freeze)
		Int_t fFreezeTick; //id of the UC_FreezeCycle that spawned us
		Int_t fT; //id of the UC_FreezeCycle that spawned us

		Int_t fCRESTId; //global ID of the detector element that spawned us
		Int_t fTubeNum;  //the rest of these should look familiar...
		Int_t fADC0;
		Int_t fADC1;
		Int_t fTDC;
		Bool_t fIsVeto;
		Int_t fBusId;
		Int_t fSTACId;
		Int_t fPMTId;
		Float_t fXLoc;
		Float_t fYLoc;
		Float_t fZLoc;

		Bool_t InitFromEvent(UC_Event* evt, Int_t idx, CRESTComponent* detector);

		ClassDef(CRESTHit, 2)   // Eve/CREST representation of a CREST detector component.
};

#endif 
