#ifndef CRESTMULTIVIEW_H
#define CRESTMULTIVIEW_H
// Multi-view (3d, rphi, rhoz) service class using EVE Window Manager.
// Author: Matevz Tadel 2009
// Copied by JGennaro

#include <TEveManager.h>

#include <TEveViewer.h>
#include <TGLViewer.h>

#include <TEveScene.h>

#include <TEveProjectionManager.h>
#include <TEveProjectionAxes.h>

#include <TEveBrowser.h>
#include <TEveWindow.h>

// CRESTMultiView
//
// Structure encapsulating standard views: 3D, r-phi and rho-z.
// JG modified to show 3D view, 2D view of crystals, and 3D view of veto system.
// Includes scenes and projection managers.

class CRESTMultiView
{
	public:
   //TEveProjectionManager *fVetoMgr;
   TEveProjectionManager *fCrystalMgr;
   TEveViewer            *f3DView; //has no projection manager
   TEveViewer            *fVetoView;//has no projection manager. How can we draw all vetos in 2D?
   TEveViewer            *fCrystalView; //has a 2D projection manager.
   TEveScene             *fVetoGeomScene; //just the veto system
   TEveScene             *fCrystalGeomScene; //just the crystal system
   TEveScene             *fVetoEventScene; //just the veto part of the event?
   TEveScene             *fCrystalEventScene;//just the crystal part of the event?

   CRESTMultiView();
   void SetDepth(Float_t d);
   void ImportGeomVeto(TEveElement* el);
   void ImportGeomCrystal(TEveElement* el);
   void ImportEventVeto(TEveElement* el);
   void ImportEventCrystal(TEveElement* el);
   void DestroyEventVeto();
   void DestroyEventCrystal();

		ClassDef(CRESTMultiView, 1)   // 3-window view of instr, veto and crystal.
};

#endif
