/*CRESTVetoPaddle.h
* Hi - I'm a CREST VetoPaddle module.  I have an X and a Y location. I store a vector of hits which is emptied every time we move from event to event. I live in a hierarchy of CRESTComponents owing to our common heritage as TEveGeoShape's.
*/

#ifndef CRESTVETOPADDLE_H
#define CRESTVETOPADDLE_H

#include "Riostream.h"
#include "TROOT.h"
#include "CRESTComponent.h"
#include "TH1F.h"

class UC_Event;

//_______________________________________________________________________________
class CRESTVetoPaddle : public CRESTComponent 
{
	private:
		//Important Inherited Members - - - - - - - - - - - - - - - - - - - - - - - - - - 
		//Int_t fCRESTId; //global ID of this detector element. 
		//Int_t fEventMember; //True if member of the active event; false otherwise.  This must be controlled by the event loading process - the components have no internal way of figuring this out. Should not be set/unset directly but by the Bool_t ProcessEvent member function. Tubes should return 1 or 0; paddles can return 2, 1 or 0. 2 = a paddle pair.
		//End Important Inherited Members - - - - - - - - - - - - - - - - - - - - - - - - - - 
		//vector<CRESTHit> fHits; //yet to be designed CRESTHits class. Reset after each event.
		//really not sure this should be kept here;...
	public:
		CRESTVetoPaddle(const char* name = "paddle", const char* title = "paddle", Int_t id=-1, TGeoHMatrix* trans=0, TGeoShape* shape=0, Bool_t is_assembly=kTRUE); 
		//Important Non-Overridden Inherited Members - - - - - - - - - - - - - - - - - - - - - - - - - - 
		//Int_t GetId() const;
		//Bool_t fIsAssembly;
		//End Important Inherited Members - - - - - - - - - - - - - - - - - - - - - - - - - - 

		//CRESTVetoPaddle* CloneVetoPaddle() const; //not sure why we might need this, but who knows...
		//virtual Int_t ProcessEvent(UC_Event*);  //override this - special for veto paddle.
		TString GetHighlightTooltip(); //override this - print more info than just the title.

	//side 0 pmt info
		Int_t fTubeNum0; //corresponds to that in UC_Geometry (unlike fCRESTId, which doesn't.)
		Int_t fBus0;
		Int_t fSedecim0;
		Int_t fPin0;

	//side 1 pmt info
		Int_t fTubeNum1; //corresponds to that in UC_Geometry (unlike fCRESTId, which doesn't.)
		Int_t fBus1;
		Int_t fSedecim1;
		Int_t fPin1;
		Float_t fXLoc; //these should be extracted from the translation matrix used to create the tube.
		Float_t fYLoc;
		Float_t fZLoc;
		//the below are used for hit location reconstruction along the length of the paddle. The TEveGeoShape class doesn't know which dimension we're interested in, so we should tell it.
		Int_t fLongDim; //the longest of the three dimensions. Each paddle is very rectangular, so there should be no confusion; = 1,2 or 3
		Float_t fLongDimLength; //the length of the longest of the three dimensions. Each paddle is very rectangular, so there should be no confusion
		TH1F	fTube0ADCHist; //veto tubes only have 1 adc value.
		TH1F	fTube1ADCHist;
		TH1F* GetADCHist(Int_t side); //

		ClassDef(CRESTVetoPaddle, 2)   // Eve/CREST representation of a CREST detector component.
};

#endif 
