#ifndef CRESTVIEWER_H
#define CRESTVIEWER_H
// Author: Jgennaro

#include <iostream>
#include <stdlib.h>
#include <map>
#include <vector>
#include <TGFrame.h>
#include <TRint.h>
#include <TFile.h>
#include <TROOT.h>
#include <string>
#include <TSystem.h>
#include <TGeoManager.h>
#include <TGeoPhysicalNode.h>
#include <TMath.h>
#include <TEveManager.h>
#include <TEveGeoNode.h>
#include <TEveBrowser.h>
#include <TEveScene.h>
#include <TEveTrans.h>
//#include <TEvePointSet.h> 
#include <TEveBoxSet.h> //UC_Events representation in Eve is an array of colored/sized boxes.
#include <TEveRGBAPalette.h> //UC_Events representation in Eve is an array of colored/sized boxes.
#include <TGButton.h>
#include <vector>
#include <RQ_OBJECT.h>
#include <TEveViewer.h>
#include <TGLViewer.h>
#include <TTree.h>
#include <TVector3.h>
#include <TPRegexp.h> //perl regular expression matching for TStrings - cool!
#include "CRESTDetector.h"
#include "CRESTNavHandler.h"
#include "CRESTMultiView.h"
#include "CRESTComponent.h"
#include "CRESTCrystal.h"
#include "CRESTCrystalView.h"
#include "CRESTVetoPaddle.h"
#include "CRESTVetoPaddleView.h"
#include "CRESTHit.h"
#include "UC_Geometry.h" //for TubeNum deliciousness!
#include "UC_Event.h"

#ifndef _CONSTANTS_H_
#include "constants.h"
#endif

// CRESTViewer
//
// GUI class which creates and interacts with the eve main window and sub windows

class CRESTViewer : public TGMainFrame 
{
	public:

		CRESTViewer();
		CRESTViewer(const TGWindow* p, UInt_t w, UInt_t h);
		~CRESTViewer();
		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		//Geometry members - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		map<Int_t,Int_t> fTubeIdMap; //key value = fTubeNum; element value = fCRESTId. 
		//map<Int_t,CRESTComponent*> fComponentIdMap; //key value = fTubeNum; element value = a CRESTComponent*
		vector<CRESTComponent*> fComponentVector; //every detector component gets stuffed into here!
		UC_Geometry* 					fGeom; //some useful geometry info/functions from UC
		void init_geometry(); //make the detector, load it into eve geometry scene
		void recursively_descend(CRESTComponent* pe,Int_t id, Int_t& global_id, Int_t& paddle_id); //helper for init_geo
		//void ImportGeomVeto(TEveElement* el); //loads only veto elements into multiview's veto geo scene
		//void ImportGeomCrystal(TEveElement* el); //loads only crystal elements into multiview's crystal geo scene
		CRESTDetector* fDetector; //the CREST detector.
		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		// - - Navigation and I/O members
		CRESTNavHandler* 			fHandler; //handles input/output and event selection
		void load_event(); //take the current event from fHandler and load it into display
		void event_back();
		void event_forward();
		void clear_event();  //destroy current event members, reset to a nice state for next event.
		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		// - - EVE event visualization members - - - - - - - - - - - - - - - - - - - - - - - - - -
		TEveBoxSet* fCrystalHitBoxSet; //draws pretty boxes on top of crystals with hits.
		TEveRGBAPalette* fCrystalPalette; //color helper for fCHBS
		//TEveBoxSet* fVetoHitBoxSet;
		//TEveRGBAPalette* fVetoPalette;
		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		// - - GUI members - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		void init_eve();
		void init_input();
		void init_gui();
		void make_control_tab();
		CRESTMultiView* 			fMultiview; //3-window pack
		//FIXME: void make_cut_tab();
		//FIXME: banana view
		//FIXME: selected ADC view.
		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


		ClassDef(CRESTViewer, 1)   // CREST Monitoring & Event Display: CrestMed
};

#endif
