///////////////////////////////////////////////////////////////////////////////
// $Id$
//
// Filename : UC_Event.h
// Created  : Wed Mar 26 06:42:19 2008 by S.P. Wakely
///////////////////////////////////////////////////////////////////////////////
#ifndef _UC_Event_H_
# define _UC_Event_H_

#include <stdint.h>
#include <string>
#include <vector>

#include <TObject.h>

#include "UC_Geometry.h"
//#include "UC_Util.h"

//so sue me.... I don't like typing std:: a million times a day
using namespace std;  

class UC_Event : public TObject 
{
  public:
  UC_Event();
  virtual ~UC_Event();
  
  ////////////////////////////////////////////////////////
  // Event Identifiers
  double	fT;	         // Time of event
  uint32_t      fFreezeTick;     // The freeze tick of the parent freeze cycle 

  int32_t       fEventCount;     // event counter starting from 0
  int32_t	fEventId;       // Event id within the freeze

  //GPS Info
  
  // other info
  uint32_t   	 fWordCount;  	 // How many data words in this event?
  uint32_t   	 fWordCountSaved;
  int16_t        fErrCount;
  uint32_t       fDeadTime;
  uint32_t	 fWatchdogCount;
  
  //fixme: hitcount, unique-tube-hitcount, full hit vectors: need separate variables;
  Int_t     	fHitCount;    //Note... do not change these comments or
  			      //data types.  They are needed for ROOT
  int16_t     	*fTubeNum;    //[fHitCount] indexed!
  int16_t     	*fTubeHits;   //[fHitCount] indexed! size of this is actually uniqHitCount
  float	      	*fADC0;       //[fHitCount] indexed!
  float	      	*fADC1;       //[fHitCount] indexed!
  float	      	*fTDC;        //[fHitCount] indexed!
  float		*fTHold;      //[fHitCount] indexed! time since hold
  int8_t        *fWithADC;    //[fHitCount] indexed!
  int8_t        *fIsVeto;     //[fHitCount] indexed!

  //debugging mode
  int16_t	*fBusId;	//[fHitCount] indexed!
  int16_t	*fSTACId;	 //[fHitCount] indexed!
  int16_t	*fPMTId;	 //[fHitCount] indexed!

  // interpreted data
  bool		fIsCal;	      //is this a cal event?
  int16_t      	fCalType;     //If this is a cal event, what kind is it?
  int16_t     	fCorLevel;    //Level of corrections applied
  float		fT0;	      //T0 information

  ////////////////////////////////////////////////////////
  // a cute way to return ADC values
  //
  vector<tubePair> getADC(bool isADC0);
  vector<tubePair> getTDC();
  
  ////////////////////////////////////////////////////////
  // Some methods, for love.
  void   setDataSize(uint16_t size); // allocate memory for the event.
  void   clearData();		     // delete event info
  void 	 fillRandom(double T);	     // fill this event w/ random info
  string dumpInfo(bool p=true);	     // make some info to stdio


  ClassDef(UC_Event, 4)  //local debugging status
};

#endif  //_UC_Event_H_
