///////////////////////////////////////////////////////////////////////////////
// $Id$
//
// Filename : UC_Geometry.h
// Created  : Wed Mar 26 08:36:07 2008 by S.P. Wakely
///////////////////////////////////////////////////////////////////////////////
#ifndef _UC_GEOMETRY_H_
#define _UC_GEOMETRY_H_

#include <string>
#include <map>

using namespace std;

///////////////////////////////////////////////////////////////////////////////
/// Singleton class with all the geometry info in it.
/// We do this with lazy initialization - not thread safe!
///
///////////////////////////////////////////////////////////////////////////////

struct tubePair{
  int   tnum;
  float tval;
};

class UC_Geometry {
  public:
  enum Config {CONUS, POLAR, TEST};  //the possible types of geometry!
  
  // A private constructor protects from multiple instances
  private:
    UC_Geometry(Config);
  public:
    virtual ~UC_Geometry();
  
  private:
  static UC_Geometry *theGeometry;
  public:
  static UC_Geometry* getInstance();
  static UC_Geometry* getInstance(Config);

  public:
  ///////////////////////////////////
  // The detector info
  Config fConfig;       // CONUS or Polar?
  int	 fTubeCount;	// How many tubes?
  int	 fXRowCount;
  int	 fYRowCount;
  
  double fX0;
  double fY0;
  double fXmin;
  double fYmin;
  double fXmax;
  double fYmax;
  double fRmax;
  
  double fXPos[1078];
  double fYPos[1078];
  bool   fFake[1078];

  int	 fTubeID[1078];
  int	 fBUSAddr[1078];
  int	 fSTACAddr[1078];
  int	 fChannelAddr[1078];
  int	 fPaddleID[1078];
  int	 fTubeGUIIDForVeto[4][2][16]; //is it better this way or circulating?

  int	 fPaddleCount;	// How many paddles?
  double fPaddleXPos[27][2];
  double fPaddleYPos[27][2];

  // generate a tube ID from all the addressing
  int getTubeID(bool is_veto, int addrBus, int addSTAC, int addTube);
  // what addresses correspond to this tube?
  int getTubeAddress(int tubeID, int &addrBus, int &addSTAC, int &addTube);
  
  static map<Config, string> fConfigNames;
  
  
};

#endif  //_UC_GEOMETRY_H_
