// Author: J Gennaro for CREST 10/11, AA

//see http://edg.uchicago.edu/projects/CREST/mech_files/ for some helpful pictures
//Also, thanks to S.Nutter and his amazingly detailed Crestdet7 detector model.

#ifndef CONSTANTS_H
#define CONSTANTS_H

//______________________________________________________________________
//*** this section are just numbers *** //
const int NUM_PMT_X = 4;
const int NUM_PMT_Y = 4;
const int NUM_CRYSTAL_BUS = 8;
const int NUM_SEDECIM_PER_BUS = 8;

const int NUM_TOP_PADDLES = 8;
const int NUM_BOT_PADDLES = 9;

const float PI = 3.14159;

const int VETO_TRANS = 80;

//______________________________________________________________________
//****this section are all lengths *** //
//****all lengths are input in inches, but converted to cm ****//
const float INCH_2_CM = 1.0/0.3937;
const float VETO_THICKNESS = 0.50*0.5;

//put the origin at the very center of the instrument - not in the corner.
const float INSTRUMENT_ZER0_X = 0;
const float INSTRUMENT_ZER0_Y = 0;
const float INSTRUMENT_ZER0_Z = 0;
const float INSTRUMENT_MARGIN = 0.0;

const float YAxisBase = INCH_2_CM*96.5; 
const float XAxisBase = INCH_2_CM*106.5;

//sedecii inside buses
const float SED_X_DIM = INCH_2_CM*11.800*0.5;
const float SED_Y_DIM = INCH_2_CM*11.800*0.5;
//const float SED_Z_DIM = INCH_2_CM*(7.480 + 2.00)*0.5;
//PMT Module Height from crestdet7
//8.015*2.54 cm = 20.36 cm
const float SED_Z_DIM = INCH_2_CM*8.015*0.5;
const float BOTTOM_VETO_Z_POS = -SED_Z_DIM-23.0 -VETO_THICKNESS;

//tubes inside sedecii - for now ignore the lead underwear
const float TUBE_XY_RADIUS = INCH_2_CM*(2.380)*0.5; //inside the lead
const float TUBELEAD_XY_RADIUS = INCH_2_CM*(2.700)*0.5; //outside the lead
const float TUBE_Z_DIM = INCH_2_CM*(7.480 + 2.00-1.63)*0.5;

//BaF2 crystals. For now, I model the tubes and not the crystals. Why? Because I can.
const float XTAL_XY_RADIUS = INCH_2_CM*1.972*0.5;
const float XTAL_Z_DIM = INCH_2_CM*0.813*0.5;
const float XTAL_Z_POS = (INCH_2_CM*7.480 - 0.5*0.813)*0.5;

//translations between tubes inside sedecii
const float PMT_X_MARGIN= INCH_2_CM*1.46;
const float PMT_X_GAP= INCH_2_CM*2.952;
const float PMT_Y_MARGIN= INCH_2_CM*1.46;
const float PMT_Y_GAP= INCH_2_CM*2.952;
const float PMT_Z_MARGIN= INCH_2_CM*0.37;


//buses
const float BUS_X_DIM= INCH_2_CM*11.800*0.5;

//vetoes
const float TOP_Z_DIM = VETO_THICKNESS;
const float BOT_Z_DIM = VETO_THICKNESS;

const float TOP_Y_INNER_DIM= INCH_2_CM*11.60*0.5; //half a sedecim wide
const float TOP_Y_EDGE_DIM= INCH_2_CM*11.86*0.5; //half a sedecim wide
//const float TOP_X_DIM= INCH_2_CM*11.800*4.0; //8 sedecii long (remember this is the half-length
const float TOP_X_DIM= INCH_2_CM*106.0/2.0; //from MGeske
const float TOP_Z_LOC= SED_Z_DIM + 2.0 ;
//const float BOT_Z_LOC= -1*SED_Z_DIM-2.0 ;
//this is wrong. from Crestdet7, it's:
//-CchannelWallHeighZ - SCCHeight1Z - 1.06*in + VetoScintillatorThickness/2.
//these are: 2.00in, 8.0in,1.06in,0.5cm/2
const float BOT_Z_LOC= -1*(2.00-1.06-8.0)*INCH_2_CM+0.50/2.0;

const float BOT_Y_DIM= INCH_2_CM*88.32/2.0; //from MGeske
const float BOT_X_DIM= INCH_2_CM*12.69/2.0; //from MGeske
const float BOT_Z_CENTER_OFFSET = BOT_Z_LOC-(INCH_2_CM)*(1.06+1.1+0.25+1.25+.06);
const float BOT_X_CENTER_GAP = INCH_2_CM*5.0; //from Crestdet7

const float SIDEY_X_DIM = TOP_X_DIM;
const float SIDEY_Y_DIM = VETO_THICKNESS;
const float SIDEY_Z_DIM = 10.010*INCH_2_CM*0.5;

const float SIDEX_X_DIM = VETO_THICKNESS;
const float SIDEX_Y_DIM = BOT_Y_DIM;
const float SIDEX_Z_DIM = 9.588*INCH_2_CM*0.5;

//these are actually SLANT the 'Support Cchannels' (called that in Crestdet7)
//the angle for both is 14.5 deg down from vertical (75.5 up from horizontal)
const float SLANTX_X_DIM = VETO_THICKNESS;
//const float SLANTX_Y_DIM = 88.0*INCH_2_CM + 2*3.74*INCH_2_CM; //this includes the top hat - makes things look bad though
const float SLANTX_Y_DIM = 88.0*INCH_2_CM;
const float SLANTX_Z_DIM = INCH_2_CM*7.25;
//z dimension = -(IBeamPlacementOffsetNegZ + CchannelWallHeighZ)
// = 4.0 in + 2.0 in
//const float SLANT_Z_LOC = INCH_2_CM*-6.0;
const float SLANT_Z_LOC = -SED_Z_DIM-INCH_2_CM*6.0;
const float SLANT_ANGLE = 14.0; //degrees

//const float SLANTY_X_DIM = 42.6*INCH_2_CM+ 2*3.72*INCH_2_CM;
const float SLANTY_X_DIM = 42.6*INCH_2_CM;
const float SLANTY_Y_DIM = VETO_THICKNESS;
const float SLANTY_Z_DIM = INCH_2_CM*6.0;
//XAxisBase/4 + IBeamFlangeWidthX/8 - VetoScintThickness/16
const float SLANTY_X_LOC = XAxisBase/4.0 + INCH_2_CM*5.0/8.0 - VETO_THICKNESS/16.0;
//YAxisBase/2 - SCTBWidth1X/2 -VETO_THICKNESS/2; 
//SCTB = SupportChannelWidth - SupportChannelThickness = 3.0 in + .350 in
const float SLANTY_Y_LOC = YAxisBase/2.0 - INCH_2_CM*3.0/2.0 - VETO_THICKNESS/2.0;
//SCCWidth1X = SupportChannelThickness = 0.350 in
//VTB = cos(76)*7.25in = 7.04 in
const float SLANTX_X_LOC = XAxisBase/2.0 - INCH_2_CM*0.350 - INCH_2_CM*7.04/2.0 - VETO_THICKNESS/2.0;
//can just take the inverse of this to send it the other direction in X
const float SLANTX_Y_LOC = 0.0;

//these lengths define a size in the unrotated coordinate system; would still need to rotate it to match reality
//they are slanted so that the top edge is further from the center of the instrument then the bottom edge
//so the X paddles look like this, in the YZ plane:
/*  -Y <<<-outside the instrument                |  inside ->>> +Y
*   <<<-outside the instrument \                |  inside ->>>
*   <<<-outside the instrument  \               |  inside ->>>
*   <<<-outside the instrument   \              |  inside ->>>
*   <<<-outside the instrument    \             |  inside ->>>
*   <<<-outside the instrument     \            |  inside ->>>
*   <<<-outside the instrument      \           |  inside ->>>
*   <<<-outside the instrument       \          |  inside ->>>
* so the AXIS of rotation is the X axis. The rotation vector points towards positive X in the right hand sense.
* the ANGLE of rotation is about 14 degrees.
* so the matrix should look something like:
[1 ] [0 ] [0 ]
[0 ] [c ] [s ]  (c = cos(theta), s = sin(theta), theta= 14 degrees
[0 ] [-s ] [c ]
* in ROOT, this is represented by a TGeoRotation with phi=0, theta=14,psi=0
* For the ypaddle,s theta=0 and psi = 14. Let's see how this works in 3D.
*/

//______________________________________________________________________
//now let's AGAIN input the veto addressing information.
//the order of the paddles is:
/*
0 TOP 0
1 TOP 1
2 TOP 2
3 TOP 3
4 TOP 4
5 TOP 5
6 TOP 6
7 TOP 7
9 Bot 0
10 Bot 1
11 Bot 2
12 Bot 3
13 Bot 4
14 Bot 5
15 Bot 6
16 Bot 7
17 Bot 8
18 SIDE -X
19 SIDE +X
19 SIDE -Y
21 SIDE +Y
22 Inside -X
23 Inside +X //has A side on -Y, B side on +Y
24 IY2 //has A side on +X, B side on -X //generally A side is towards teh center.
25 IY3 //has B side on +X, A side on -X
26 IY4 //has A side on +X, B side on -X
27 IY1 //has B side on +X, A side on -X
*/

// array columns are tube, bus, stac, pin
//each paddle takes two rows
const static Int_t Addresses[54][3] = {
	{3,	2,	4},
	{4,	2,	3},
	{3,	2,	2},
	{4,	2,	7},
	{3,	2,	0},
	{4,	2,	1},
	{3,	2,	1},
	{4,	2,	2},
	{2,	2,	1},
	{1,	2,	0},
	{2,	2,	2},
	{1,	2,	1},
	{2,	2,	3},
	{1,	2,	2},
	{2,	1,	7},
	{1,	2,	7},
	{3,	1,	0},
	{2,	1,	0},
	{3,	1,	1},
	{2,	1,	1},
	{3,	1,	2},
	{2,	1,	2},
	{3,	1,	4},
	{2,	1,	3},
	{4,	1,	5},
	{2,	2,	5},
	{4,	1,	0},
	{1,	1,	4},
	{4,	1,	1},
	{1,	1,	1},
	{4,	1,	2},
	{1,	1,	2},
	{4,	1,	3},
	{1,	1,	3},
	{3,	2,	5},
	{2,	2,	7},
	{4,	2,	9},
	{1,	2,	8},
	{3,	1,	7},
	{4,	2,	6},
	{2,	2,	4},
	{1,	2,	5},
	{3,	1,	6},
	{2,	1,	5},
	{4,	2,	4},
	{1,	2,	6},
	{3,	2,	7},
	{3,	2,	6},
	{4,	1,	4},
	{4,	1,	6},
	{2,	1,	6},
	{1,	1,	5},
	{1,	1,	7},
	{1,	1,	6},
};

#endif // CONSTANTS_H
