
#include "CRESTComponent.h"
//#include "TEveElementList.h"
#include <iostream>
#include <TGeoMatrix.h>
#include <TGeoShape.h>
#include <TEveTrans.h>
#include <TVector3.h>

class TEveElement;

ClassImp(CRESTComponent)

CRESTComponent::CRESTComponent(const char* name, const char* title, Int_t id, TGeoHMatrix* trans, TGeoShape* shape, Bool_t is_assembly) : TEveGeoShape(name,title)
{
	cout << "Making a CRESTComponent ";
	fName = name;
	fTitle = title;
	fCRESTId = id;
	fIsAssembly = is_assembly;
	if(is_assembly) cout << " (as an assembly, not to be drawn) ";
	else cout << " (not as an assembly, to be drawn) ";
	cout << "With name " << name << ", title " << title << " and CRESTId " << fCRESTId << endl;
	//set the location
	RefMainTrans().SetFrom(*trans);
	TEveTrans ttrans = RefMainTrans();
	TVector3 tvec = ttrans.GetPos();
	fXLoc = tvec.X();
	fYLoc = tvec.Y();
	fZLoc = tvec.Z();
	//set the shape
	SetShape( shape);
	cout << "\t setting component's position to " << fXLoc << " : " << fYLoc << endl;
	//SetRnrSelfChildren(kFALSE,kTRUE); //don't draw assembly volumes, only physical volumes. By default we start out as an assembly.
	SetRnrSelfChildren(!fIsAssembly,kTRUE); //don't draw assembly volumes, only physical volumes.
	SetPickable(!fIsAssembly);
	if(!trans || !shape) {
		cout << "Info in <CRESTComponent::CRESTComponent>: null trans and/or shape provided." << endl;
		if(!fIsAssembly) cout << "Warning in <CRESTComponent::CRESTComponent>: AND fIsAssembly is false - this will give the GL painter fits." << endl;
	}
}

Int_t CRESTComponent::GetId() const //return the global CREST Id number of this component. Normally this will also equal the parent global id + the daughter id in the geometry hierarchy.
{
	return fCRESTId;
}
