// @(#)root/eve:$Id: TEveElementEditor.cxx 33864 2010-06-14 09:47:19Z matevz $
// Authors: Matevz Tadel & Alja Mrak-Tadel: 2006, 2007

/*************************************************************************
 * Copyright (C) 1995-2007, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

//#include "TEveElementEditor.h"
#include "TEveElement.h"
#include "TEveTransEditor.h"

#include "TVirtualPad.h"
#include "TColor.h"

#include "TGLabel.h"
#include "TGButton.h"
#include "TGNumberEntry.h"
#include "TGColorSelect.h"
#include "TGDoubleSlider.h"

#include "CRESTCrystal.h"
#include "CRESTCrystalEditor.h"
#include "CRESTCrystalView.h"

//==============================================================================
//==============================================================================
// TEveElementEditor
//==============================================================================

//______________________________________________________________________________
//
// Editor for CRESTCrystal class.

ClassImp(CRESTCrystalEditor);

//______________________________________________________________________________
CRESTCrystalEditor::CRESTCrystalEditor(const TGWindow *p,
                                     Int_t width, Int_t height,
                                     UInt_t options, Pixel_t back) :
   TGedFrame(p, width, height, options | kVerticalFrame, back),

   fRE           (0),
   fVFrame       (0),
   //fPreLabel     (0),
   //fRnrSelf      (0),
   //fRnrChildren  (0),
   //fRnrState     (0),
   //fMainColor    (0),
   //fTransparency (0),
   //fTrans        (0),
   fBusLabel        (0),
   fSedecimLabel        (0),
   fPinLabel        (0),
	 fHistButton(0)
{
   // Constructor.

   MakeTitle("CRESTCrystal");
   fPriority = 0;


	cout << "Creating CRESTCrystalEditor." << endl;
   fVFrame = new TGVerticalFrame(this);
   fBusLabel = new TGLabel(fVFrame, "Bus:");
   fVFrame->AddFrame(fBusLabel, new TGLayoutHints(kLHintsLeft | kLHintsTop, 1, 6, 2, 0));
   fSedecimLabel = new TGLabel(fVFrame, "Sedecim:");
   fVFrame->AddFrame(fSedecimLabel, new TGLayoutHints(kLHintsLeft | kLHintsTop, 1, 6, 2, 0));
   fPinLabel = new TGLabel(fVFrame, "Pin:");
   fVFrame->AddFrame(fPinLabel, new TGLayoutHints(kLHintsLeft | kLHintsTop, 1, 6, 2, 0));
   AddFrame(fVFrame, new TGLayoutHints(kLHintsTop, 0, 0, 0, 0));

   fHistButton = new TGPictureButton(fVFrame, gClient->GetPicture("draw_t.xpm"));
   fHistButton->Connect
      ("Clicked()",
       "CRESTCrystalEditor", this, "DoHist()");
   //fHistButton->Resize(20,20);
   fVFrame->AddFrame(fHistButton, new TGLayoutHints(kLHintsLeft | kLHintsTop, 1, 6, 2, 0));

   AddFrame(fVFrame, new TGLayoutHints(kLHintsTop, 0, 0, 0, 0));
}

/******************************************************************************/

//______________________________________________________________________________
void CRESTCrystalEditor::SetModel(TObject* obj)
{
   // Set model object.

   fRE = dynamic_cast<CRESTCrystal*>(obj);
	cout << "Set model of CRESTCrystalEditor to " << fRE->GetElementTitle() << endl;

	//fBusLabel->UnmapWindow();
	fBusLabel->MapWindow();
	fSedecimLabel->MapWindow();
	fPinLabel->MapWindow();
	/*
   fPreLabel   ->UnmapWindow();
   fRnrSelf    ->UnmapWindow();
   fRnrChildren->UnmapWindow();
   fRnrState   ->UnmapWindow();
   if (fRE->CanEditElement()) {
      fPreLabel->MapWindow();
      if (fRE->SingleRnrState()) {
         fRnrState->SetState(fRE->GetRnrState() ? kButtonDown : kButtonUp);
         fRnrState->MapWindow();
      } else {
         fRnrSelf->SetState(fRE->GetRnrSelf() ? kButtonDown : kButtonUp);
         fRnrChildren->SetState(fRE->GetRnrChildren() ? kButtonDown : kButtonUp);
         fRnrSelf->MapWindow();
         fRnrChildren->MapWindow();
      }
   }

   if (fRE->CanEditMainColor()) {
      fMainColor->SetColor(TColor::Number2Pixel(fRE->GetMainColor()), kFALSE);
      fMainColor->MapWindow();
   } else {
      fMainColor->UnmapWindow();
   }
   if (fRE->CanEditMainTransparency()) {
      fTransparency->SetNumber(fRE->GetMainTransparency());
      fTransparency->MapWindow();
   } else {
      fTransparency->UnmapWindow();
   }
   if (fRE->CanEditMainTrans()) {
      fTrans->SetModel(fRE->PtrMainTrans());
      fTrans->MapWindow();
   } else {
      fTrans->UnmapWindow();
   }
	 */

	 TString content;
	 content = TString::Format("Bus: %d",fRE->fBus);
	 fBusLabel->SetText(content);
	 content = TString::Format("Sedecim: %d",fRE->fSedecim);
	 fSedecimLabel->SetText(content);
	 content = TString::Format("Pin: %d",fRE->fPin);
	 fPinLabel->SetText(content);
   fVFrame->Layout();
}

/******************************************************************************/

void CRESTCrystalEditor::DoHist()
{
   // Slot for drawing this crystal's ADC histogram. TBD: Total, rolling or current event only? For now there is only total.

   cout << "Drawing ADC Histogram for crystal " << fRE->GetElementTitle() << endl;
	 CRESTCrystalView* view = new CRESTCrystalView();
	 view->SetElement(fRE);
	 view->ImportADCHists();
	 view->DrawHist(3);
   Update();
}
/*
//______________________________________________________________________________
void CRESTCrystalEditor::DoRnrSelf()
{
   // Slot for RnrSelf.

   fRE->SetRnrSelf(fRnrSelf->IsOn());
   Update();
}

//______________________________________________________________________________
void CRESTCrystalEditor::DoRnrChildren()
{
   // Slot for RnrChildren.

   fRE->SetRnrChildren(fRnrChildren->IsOn());
   Update();
}

//______________________________________________________________________________
void CRESTCrystalEditor::DoRnrState()
{
   // Slot for RnrState.

   fRE->SetRnrState(fRnrState->IsOn());
   Update();
}

//______________________________________________________________________________
void CRESTCrystalEditor::DoMainColor(Pixel_t color)
{
   // Slot for MainColor.

   fRE->SetMainColorPixel(color);
   Update();
}

//______________________________________________________________________________
void CRESTCrystalEditor::DoTransparency()
{
   // Slot for Transparency.

   fRE->SetMainTransparency((Char_t)(fTransparency->GetNumber()));
   Update();
}
*/
