// Author: Bertrand Bellenot   22/08/02

/*************************************************************************
* Copyright (C) 1995-2002, Bertrand Bellenot.                           *
* All rights reserved.                                                  *
*                                                                       *
* For the licensing terms see the LICENSE file.                         *
*************************************************************************/

#include <stdlib.h>
#include <iostream>
#include "TROOT.h"
#include "TMath.h"
#include <cmath>

#ifndef CRESTDETECTOR_H
#include "CRESTDetector.h"
#endif

#include "TGeoPhysicalNode.h" 

using namespace std;
//______________________________________________________________________________
//
// CRESTDetector class implementation
//______________________________________________________________________________


ClassImp(CRESTDetector)

//______________________________________________________________________________
CRESTDetector::CRESTDetector() 
{
	// Default constructor.
}

//______________________________________________________________________________
CRESTDetector::~CRESTDetector() 
{
	// Destructor.
}

//______________________________________________________________________________
void CRESTDetector::Init()
{
	// Initialize detector with material and dimensions

new TGeoManager("CRESTDetector", "CRESTDetector");

	//here we have to define the constants
	//Bus constants
	//Sedecim constants
	//Veto constants
	//Derived constants:
		//Instrument dimension:
		fDimX = 2000;
		fDimY = 2000;
		fDimZ = 500;
	//these translations define the relationships between instrument components
	TGeoTranslation PMT_in_sed_X_margin_trans (PMT_X_MARGIN,						 0.,			0.);
	TGeoTranslation PMT_in_sed_Y_margin_trans (0., 				PMT_Y_MARGIN,			0.);
	TGeoTranslation PMT_in_sed_X_trans (PMT_X_GAP,						 0.,			0.);
	TGeoTranslation PMT_in_sed_Y_trans (0., 				PMT_Y_GAP,			0.);
	TGeoTranslation PMT_in_sed_Z_trans (0., 				0.,				TUBE_Z_DIM+PMT_Z_MARGIN);
	TGeoTranslation XTAL_in_sed_Z_trans (0., 				0.,				XTAL_Z_POS);
	TGeoTranslation to_sed_corner_trans (-SED_X_DIM,-SED_Y_DIM,-SED_Z_DIM);

	TGeoTranslation sed_in_bus_X_trans (SED_Y_DIM*2.0,0.,			0.);
	TGeoTranslation to_bus_start_trans (-SED_X_DIM*4.0*2.0 + SED_X_DIM, 0., 0.);
	TGeoTranslation to_instr_side_Y_trans (0.,-SED_Y_DIM*4.0*2.0+SED_Y_DIM, 0.);
	TGeoTranslation to_instr_side_X_trans (-SED_X_DIM*4.0*2.0+SED_X_DIM,0., 0.);
	TGeoTranslation bus_in_instr_Y_trans (0, SED_Y_DIM*2.0, 0.);

	//veto trans
	TGeoTranslation top_in_instr_Y_edge_trans (0, TOP_Y_EDGE_DIM*2.0, 0.);
	TGeoTranslation top_in_instr_Y_inner_trans (0, TOP_Y_INNER_DIM*2.0, 0.);
	TGeoTranslation to_veto_edge_Y (0, -(TOP_Y_INNER_DIM*6.0+2.0*TOP_Y_EDGE_DIM), 0.);
	TGeoTranslation half_edge_paddle_Y (0.,TOP_Y_EDGE_DIM, 0.);
	TGeoTranslation to_veto_edge_X (-BOT_X_DIM*7.0-BOT_X_CENTER_GAP*0.5,0., 0.);
	TGeoTranslation top_Z_trans (0., 0., SED_Z_DIM+2); //2 cm above top of sedecim?
	TGeoTranslation bot_in_instr_X_trans (BOT_X_DIM*2.0, 0., 0.);
	TGeoTranslation bot_in_instr_X_half_trans (BOT_X_DIM, 0., 0.);
	TGeoTranslation bot_in_instr_X_gap_trans (BOT_X_CENTER_GAP/2.0, 0., 0.);
	//TGeoTranslation bot_Z_trans (0., 0., -30.0); //waiting for word from veto bros on this
	TGeoTranslation bot_Z_trans (0., 0., BOTTOM_VETO_Z_POS);
	//TGeoTranslation bot_Z_trans_boss (0., 0., BOT_Z_DIM); //for the slightly sunken center bottom paddle?
	TGeoTranslation bot_Z_trans_boss (0., 0., -4.0); //for the slightly sunken center bottom paddle?

	//inside slanted veto's (smallest vetos, most trouble!)
	TGeoRotation slantX_rot_Y,slantY_rot_X;
	slantX_rot_Y.RotateY(14.0);
	slantY_rot_X.RotateX(14.0);
	TGeoTranslation slantY_centerX (SLANTY_X_LOC,0.,0.); //+ and - this for the two sets of two slantY paddles
	TGeoTranslation slantY_centerY (0.,SLANTY_Y_LOC,0.); //there are four slantY paddles, will take this trans, and its inverse
	TGeoTranslation slantX_centerX (SLANTX_X_LOC,0.,0.); //+ and - this for the two slantX paddles
	TGeoTranslation slantX_centerY (0.,SLANTX_Y_LOC,0.);  //+ and - this for the two slantX paddles
	TGeoTranslation slant_centerZ (0.,0.,SLANT_Z_LOC);
	/*
	TGeoMaterial* Paddlium = new TGeoMaterial("Pa") //veto paddle scintillator
	TGeoMaterial* Crystalium = new TGeoMaterial("Cry")// baf2
	TGeoMaterial* Crestium  = new TGeoMaterial("Crst") //an innovative metal that promotes fresh breath
	*/
	TGeoMaterial *Voidium= new TGeoMaterial("Void_Material");
	TGeoMedium *void_medium = new TGeoMedium("Void_Medium",1,Voidium);

	fGVTop = gGeoManager->MakeBox("TOP",void_medium,SED_X_DIM*8.0+INSTRUMENT_MARGIN,SED_Y_DIM*8.0+INSTRUMENT_MARGIN,SED_Z_DIM*2.0+INSTRUMENT_MARGIN);
	fGVTop->SetVisibility(kFALSE);
	gGeoManager->SetTopVisible(); //usually the TOP is invisible
	gGeoManager->SetTopVolume(fGVTop);

	//elementary starting point:  a Sedecim
	TGeoVolume *sedecim = new TGeoVolumeAssembly("SEDECIM");
	sedecim->SetVisibility(kFALSE);

	//TGeoVolume *tube = gGeoManager->MakeTube("TUBE",void_medium,0.0,TUBE_XY_RADIUS,TUBE_Z_DIM);
	//important choice here. do we want the crystals or the pmts or both? I thinkw we want the crystals..
	TGeoVolume *xtal = gGeoManager->MakeTube("XTAL",void_medium,0.0,XTAL_XY_RADIUS,XTAL_Z_DIM);
	xtal->SetLineColor(kWhite);
	xtal->SetTransparency(90);
	//loop over the four rows and columns of the sedicim, add xtal at each point.
	TGeoHMatrix the_pmt_trans;
	the_pmt_trans = to_sed_corner_trans;
	//the_pmt_trans *= PMT_in_sed_Z_trans; 
	the_pmt_trans *= XTAL_in_sed_Z_trans;  //crystal position instead of tube position
	the_pmt_trans *= PMT_in_sed_Y_margin_trans;
	the_pmt_trans *= PMT_in_sed_X_margin_trans;
	int nxtal=0;
	for (int iy=0;iy<NUM_PMT_Y;iy++) {
		for (int ix=0;ix<NUM_PMT_X;ix++) {
			TGeoHMatrix *ph = new TGeoHMatrix(the_pmt_trans);
			sedecim->AddNode(xtal,nxtal,ph);
			nxtal++;
			the_pmt_trans*=PMT_in_sed_Y_trans;
		}
		for(int j=0;j<NUM_PMT_Y;j++) the_pmt_trans*=(PMT_in_sed_Y_trans.Inverse());
		the_pmt_trans*=PMT_in_sed_X_trans;
	}
	//fGVTop->AddNode(sedecim,1,0); //testing only!

	//now make the bus assembly, which is an assembly of 'sedicim' s
	TGeoHMatrix the_sed_trans;
	the_sed_trans = to_bus_start_trans;
	TGeoVolume *bus = new TGeoVolumeAssembly("BUS");
	bus->SetLineColor(kRed);
	bus->SetVisibility(kFALSE);
	for (int ised=0;ised<NUM_SEDECIM_PER_BUS;ised++) {
		TGeoHMatrix *ph = new TGeoHMatrix(the_sed_trans);
		bus->AddNode(sedecim,ised,ph);	
		cout << "Placed sed coords: " << ph->GetTranslation()[0] << ", " <<  ph->GetTranslation()[1] << ", " <<  ph->GetTranslation()[2] << endl;
		the_sed_trans *= sed_in_bus_X_trans;
	}
	//fGVTop->AddNode(bus,1,0); //testing only!

	//Now, add the buses to the top volume.
	TGeoHMatrix the_bus_trans;
	the_bus_trans = to_instr_side_Y_trans;
	for (int ibus=0;ibus<NUM_CRYSTAL_BUS;ibus++) {
		TGeoHMatrix *ph = new TGeoHMatrix(the_bus_trans);
		fGVTop->AddNode(bus,ibus,ph);
		cout << "Placed bus coords: " << ph->GetTranslation()[0] << ", " <<  ph->GetTranslation()[1] << ", " <<  ph->GetTranslation()[2] << endl;
		the_bus_trans *= bus_in_instr_Y_trans;
	}

	//Great. This all works; now let's add some veto paddles.
	//Let's have all the veto subsystems be added directly to the top volume to keep the node paths short.
	//First, let's make the basic shapes for the veto paddles.
	TGeoVolume *topveto = new TGeoVolumeAssembly("TOPVETO");
	TGeoVolume *top_inner_paddle = gGeoManager->MakeBox("PADDLE",void_medium,TOP_X_DIM,TOP_Y_INNER_DIM,TOP_Z_DIM);
	TGeoVolume *top_edge_paddle = gGeoManager->MakeBox("PADDLE",void_medium,TOP_X_DIM,TOP_Y_EDGE_DIM,TOP_Z_DIM);
	TGeoHMatrix the_top_trans;
	the_top_trans = to_veto_edge_Y;
	//now move forward half of an edge padle.
	the_top_trans *= half_edge_paddle_Y;
	for (int itop=0;itop<NUM_TOP_PADDLES;itop++) {
		TGeoHMatrix *ph = new TGeoHMatrix(the_top_trans);
		if(itop==0 || itop==NUM_TOP_PADDLES-1) { //edge
			top_edge_paddle->SetLineColor(kGreen);
			top_edge_paddle->SetTransparency(VETO_TRANS);
			topveto->AddNode(top_edge_paddle,itop,ph);
			the_top_trans *= top_in_instr_Y_edge_trans;
		} else {
			top_inner_paddle->SetLineColor(kGreen);
			top_inner_paddle->SetTransparency(VETO_TRANS);
			topveto->AddNode(top_inner_paddle,itop,ph);
			the_top_trans *= top_in_instr_Y_inner_trans;
		}
	}
	the_top_trans.Clear();
	the_top_trans = top_Z_trans;
	//the_top_trans *= to_instr_side_Y_trans;
	//move it up just a tad
	TGeoHMatrix *ph = new TGeoHMatrix(the_top_trans);
	//topveto->SetVisibility(kFALSE);
	//topveto->SetLineColor(kWhite);
	fGVTop->AddNode(topveto,0,ph);

	TGeoVolume *botveto = new TGeoVolumeAssembly("BOTVETO");
	TGeoVolume *bot_paddle = gGeoManager->MakeBox("PADDLE",void_medium,BOT_X_DIM,BOT_Y_DIM,BOT_Z_DIM);
	//TGeoVolume *bot_narrow_paddle = gGeoManager->MakeBox("PADDLE",void_medium,BOT_X_NARROW_DIM,BOT_Y_DIM,TOP_Z_DIM); //no! all paddles seem to be normal size.
	TGeoHMatrix the_bot_trans;
	the_bot_trans = to_veto_edge_X;
	for (int ibot=0;ibot<NUM_BOT_PADDLES;ibot++) { //paddles
		TGeoHMatrix *ph = new TGeoHMatrix(the_bot_trans);
		bot_paddle->SetLineColor(kGreen);
		bot_paddle->SetTransparency(VETO_TRANS);
		botveto->AddNode(bot_paddle,ibot,ph);
		if(ibot==3) { //do half translation in Y, allow for the gap, and the small Z translation
			the_bot_trans *= bot_in_instr_X_half_trans;
			the_bot_trans *= bot_in_instr_X_gap_trans;
			the_bot_trans *= bot_Z_trans_boss;
			continue;
		} else if(ibot==4) {
			the_bot_trans *= bot_in_instr_X_half_trans;
			the_bot_trans *= bot_in_instr_X_gap_trans;
			the_bot_trans *= bot_Z_trans_boss.Inverse();
			continue;
		} else {
			the_bot_trans *= bot_in_instr_X_trans;
		}
	}
	the_bot_trans.Clear();
	the_bot_trans = bot_Z_trans;
	TGeoHMatrix *phbot = new TGeoHMatrix(the_bot_trans);
	//botveto->SetVisibility(kFALSE);
	//botveto->SetLineColor(kWhite);
	fGVTop->AddNode(botveto,0,phbot);

	//side veto system
	TGeoVolume *sideveto = new TGeoVolumeAssembly("OUTSIDEVETO");

	//the x side veto's
	TGeoVolume *x_side_paddle = gGeoManager->MakeBox("SIDEXPADDLE",void_medium,SIDEX_X_DIM,SIDEX_Y_DIM,SIDEX_Z_DIM);
	TGeoHMatrix the_side_trans;
	the_side_trans = to_veto_edge_X;
	x_side_paddle->SetLineColor(kGreen);
	x_side_paddle->SetTransparency(VETO_TRANS);
	TGeoHMatrix* physide = new TGeoHMatrix(the_side_trans);
	sideveto->AddNode(x_side_paddle,0,physide);
	the_side_trans = to_veto_edge_X.Inverse();
	x_side_paddle->SetLineColor(kTeal);
	x_side_paddle->SetTransparency(VETO_TRANS);
	physide = new TGeoHMatrix(the_side_trans);
	sideveto->AddNode(x_side_paddle,1,physide);

	//the Y side vetoes.
	TGeoVolume *y_side_paddle = gGeoManager->MakeBox("SIDEYPADDLE",void_medium,SIDEY_X_DIM,SIDEY_Y_DIM,SIDEY_Z_DIM);
	the_side_trans = to_veto_edge_Y;
	y_side_paddle->SetLineColor(kGreen);
	y_side_paddle->SetTransparency(VETO_TRANS);
	physide = new TGeoHMatrix(the_side_trans);
	sideveto->AddNode(y_side_paddle,0,physide);
	for (int i=0;i<2;i++) //get to the other side fast!
		the_side_trans *= top_in_instr_Y_edge_trans;
	for (int i=0;i<6;i++) //get to the other side fast!
		the_side_trans *= top_in_instr_Y_inner_trans;
	physide = new TGeoHMatrix(the_side_trans);
	sideveto->AddNode(y_side_paddle,1,physide);
	fGVTop->AddNode(sideveto,0,0);

	//great! now for the inside slanted veto's then let's call it a day!
	//there are 6 of these, 2 on the x side and four half length on the y side to accomodate the center ibeam.
	//first, the two x.
	TGeoVolume *slantveto = new TGeoVolumeAssembly("INSIDESLANTEDVETO");
	TGeoVolume *x_slant_paddle = gGeoManager->MakeBox("XSLANTPADDLE",void_medium,SLANTX_X_DIM/2.0,SLANTX_Y_DIM/2.0,SLANTX_Z_DIM/2.0);
	TGeoHMatrix the_slant_trans;
	the_slant_trans = slantX_centerX.Inverse();
	the_slant_trans *= slant_centerZ;
	the_slant_trans *= slantX_rot_Y;
	x_slant_paddle->SetLineColor(kRed);
	x_slant_paddle->SetTransparency(VETO_TRANS);
	TGeoHMatrix* phslantX = new TGeoHMatrix(the_slant_trans);
	slantveto->AddNode(x_slant_paddle,0,phslantX);
	the_slant_trans = slantX_centerX;
	the_slant_trans *= slant_centerZ;
	the_slant_trans *= slantX_rot_Y.Inverse();
	phslantX = new TGeoHMatrix(the_slant_trans);
	slantveto->AddNode(x_slant_paddle,1,phslantX);

	TGeoVolume *y_slant_paddle = gGeoManager->MakeBox("YSLANTPADDLE",void_medium,SLANTY_X_DIM/2.0,SLANTY_Y_DIM/2.0,SLANTY_Z_DIM/2.0);
	y_slant_paddle->SetLineColor(kYellow);
	y_slant_paddle->SetTransparency(VETO_TRANS);
	//the_slant_trans = slantY_centerX; //above veto box 1; +x +y
	//actually let's try to match the order of the existing veto map.
	the_slant_trans = slantY_centerX.Inverse(); //first is 'insidey2' at -X,-Y
	the_slant_trans *= slantY_centerY.Inverse();
	the_slant_trans *= slant_centerZ;
	the_slant_trans *= slantY_rot_X.Inverse();
	TGeoHMatrix* phslantY = new TGeoHMatrix(the_slant_trans);
	slantveto->AddNode(y_slant_paddle,0,phslantY);

	the_slant_trans = slantY_centerX; //Next, is 'insidey3' at +X,-Y
	the_slant_trans *= slantY_centerY.Inverse();
	the_slant_trans *= slant_centerZ;
	the_slant_trans *= slantY_rot_X.Inverse();
	phslantY = new TGeoHMatrix(the_slant_trans);
	slantveto->AddNode(y_slant_paddle,1,phslantY);

	the_slant_trans = slantY_centerX.Inverse(); //Now 'inside4' at -X,+Y
	the_slant_trans *= slantY_centerY;
	the_slant_trans *= slant_centerZ;
	the_slant_trans *= slantY_rot_X;
	phslantY = new TGeoHMatrix(the_slant_trans);
	slantveto->AddNode(y_slant_paddle,4,phslantY);

	the_slant_trans = slantY_centerX; //Now 'inside1' at +X, +Y
	the_slant_trans *= slantY_centerY;
	the_slant_trans *= slant_centerZ;
	the_slant_trans *= slantY_rot_X;
	phslantY = new TGeoHMatrix(the_slant_trans);
	slantveto->AddNode(y_slant_paddle,5,phslantY);


	fGVTop->AddNode(slantveto,0,0);
	//gGeoManager->SetVisOption(0); //by default only the deepest objects (or those 3 levels or deeper) are drawn; 0 forces drawing of all.
	//gGeoManager->CloseGeometry("i");
	gGeoManager->CloseGeometry();

}

