// Multi-view (3d, rphi, rhoz) service class using EVE Window Manager.
// Author: Matevz Tadel 2009

// MultiView
//
// Structure encapsulating standard views: 3D, r-phi and rho-z.
// Includes scenes and projection managers.
//
// Should be used in compiled mode.

#include "CRESTMultiView.h"
#include <iostream>

ClassImp(CRESTMultiView);

	CRESTMultiView::CRESTMultiView()
	{
	// Constructor --- creates required scenes, projection managers
	// and GL viewers.

		// Scenes
		//========
		fVetoGeomScene  = gEve->SpawnNewScene("Veto Geometry",
																		"Scene holding projected geometry for the Veto view.");
		fCrystalGeomScene  = gEve->SpawnNewScene("Crystal Geometry",
																		"Scene holding projected geometry for the Crystal view.");
		fVetoEventScene = gEve->SpawnNewScene("Veto Event Data",
																		"Scene holding projected event-data for the Veto view.");
		fCrystalEventScene = gEve->SpawnNewScene("Crystal Event Data",
																		"Scene holding projected event-data for the Crystal view.");


	// Projection managers
	//=====================
		//fVetoMgr = new TEveProjectionManager(TEveProjection::kPT_3D); //normal 3D view
		//fVetoMgr = new TEveProjectionManager(TEveProjection::kPT_RPhi); //normal 3D view
		//gEve->AddToListTree(fVetoMgr, kFALSE);
		/*
		{
			TEveProjectionAxes* a = new TEveProjectionAxes(fVetoMgr);
			a->SetMainColor(kWhite);
			a->SetTitle("Veto Axes");
			a->SetTitleSize(0.05);
			a->SetTitleFont(102);
			a->SetLabelSize(0.025);
			a->SetLabelFont(102);
			fVetoGeomScene->AddElement(a);
		}
		*/

		fCrystalMgr = new TEveProjectionManager(TEveProjection::kPT_RPhi); //top down, 2D view
		gEve->AddToListTree(fCrystalMgr, kFALSE);
		{
			TEveProjectionAxes* a = new TEveProjectionAxes(fCrystalMgr);
			a->SetMainColor(kWhite);
			a->SetTitle("Crystal Axes");
			a->SetTitleSize(0.05);
			a->SetTitleFont(102);
			a->SetLabelSize(0.025);
			a->SetLabelFont(102);
			fCrystalGeomScene->AddElement(a);
		}


		// Viewers
		//=========

		TEveWindowSlot *slot = 0;
		TEveWindowPack *pack = 0;

		slot = TEveWindow::CreateWindowInTab(gEve->GetBrowser()->GetTabRight());
		pack = slot->MakePack();
		pack->SetElementName("Multi View");
		pack->SetHorizontal();
		pack->SetShowTitleBar(kFALSE);
		pack->NewSlot()->MakeCurrent();
		f3DView = gEve->SpawnNewViewer("3D View", "");
		TGLViewer *gv = f3DView->GetGLViewer();
		gv->SetGuideState(TGLUtil::kAxesOrigin,kTRUE,kFALSE,0);
		gv->SetCurrentCamera(TGLViewer::kCameraPerspXOY);
		f3DView->AddScene(gEve->GetGlobalScene());
		f3DView->AddScene(gEve->GetEventScene());

		pack = pack->NewSlot()->MakePack();
		pack->SetShowTitleBar(kFALSE);
		pack->NewSlot()->MakeCurrent();
		fVetoView = gEve->SpawnNewViewer("Veto View", "");
		gv = fVetoView->GetGLViewer();
		gv->SetGuideState(TGLUtil::kAxesOrigin,kTRUE,kFALSE,0);
		gv->SetCurrentCamera(TGLViewer::kCameraPerspXOY);
		//fVetoView->GetGLViewer()->SetCurrentCamera(TGLViewer::kCameraOrthoXOY);
		fVetoView->AddScene(fVetoGeomScene);
		fVetoView->AddScene(fVetoEventScene);

		pack->NewSlot()->MakeCurrent();
		fCrystalView = gEve->SpawnNewViewer("Crystal View", "");
		fCrystalView->GetGLViewer()->SetCurrentCamera(TGLViewer::kCameraOrthoXOY);
		fCrystalView->AddScene(gEve->GetGlobalScene());
		//fCrystalView->AddScene(fCrystalGeomScene);
		//fCrystalView->AddScene(fCrystalEventScene);
	}

	//---------------------------------------------------------------------------
	void CRESTMultiView::SetDepth(Float_t d)// Set current depth on all projection managers.
	{
		//fVetoMgr->SetCurrentDepth(d);
		//fCrystalMgr->SetCurrentDepth(d);
	}

	//---------------------------------------------------------------------------
	void CRESTMultiView::ImportGeomVeto(TEveElement* el)
	{ 
		if(!el) {
			cout << "Error in ImportGeomVeto: got null TEveElement!" << endl;
			return;
		}
		//cout << "About to ImportElements in ImportGeomVeto with an element named " << el->GetElementName() << endl;
		//fVetoMgr->ImportElements(el, fVetoGeomScene);
		//NO! don't want a projection manager for this window - just want regular 3D view.
		fVetoGeomScene->AddElement(el);
	}

	void CRESTMultiView::ImportGeomCrystal(TEveElement* el)
	{ 
		fCrystalMgr->ImportElements(el, fCrystalGeomScene);
	}

	void CRESTMultiView::ImportEventVeto(TEveElement* el)
	{ 
		//fVetoMgr->ImportElements(el, fVetoEventScene);
		fVetoEventScene->AddElement(el);
	}

	void CRESTMultiView::ImportEventCrystal(TEveElement* el)
	{ 
		//fCrystalMgr->ImportElements(el, fCrystalEventScene);
	}

	//---------------------------------------------------------------------------

	void CRESTMultiView::DestroyEventVeto()
	{
		fVetoEventScene->DestroyElements();
	}

	void CRESTMultiView::DestroyEventCrystal()
	{
		fCrystalEventScene->DestroyElements();
	}
