// @(#)root/eve:$Id: TEveElementEditor.cxx 33864 2010-06-14 09:47:19Z matevz $
// Authors: Matevz Tadel & Alja Mrak-Tadel: 2006, 2007

/*************************************************************************
 * Copyright (C) 1995-2007, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

//#include "TEveElementEditor.h"
#include "TEveElement.h"
#include <iostream>
//#include "TEveTransEditor.h"

#include "TVirtualPad.h"
#include "TColor.h"

#include "TGLabel.h"
#include "TGButton.h"
//#include "TGNumberEntry.h"
//#include "TGColorSelect.h"
//#include "TGDoubleSlider.h"

#include "CRESTVetoPaddle.h"
#include "CRESTVetoPaddleEditor.h"
#include "CRESTVetoPaddleView.h"

using namespace std;
//==============================================================================
//==============================================================================
// TEveElementEditor
//==============================================================================

//______________________________________________________________________________
//
// Editor for CRESTVetoPaddle class.

ClassImp(CRESTVetoPaddleEditor);

//______________________________________________________________________________
CRESTVetoPaddleEditor::CRESTVetoPaddleEditor(const TGWindow *p,
                                     Int_t width, Int_t height,
                                     UInt_t options, Pixel_t back) :
   TGedFrame(p, width, height, options | kVerticalFrame, back),
   fRE           (0),
   fHFrame0       (0),
   fHFrame1       (0),
   fBusLabel0        (0),
   fSedecimLabel0        (0),
   fPinLabel0        (0),
   fTubeLabel1        (0),
   fBusLabel1        (0),
   fSedecimLabel1        (0),
   fPinLabel1        (0),
	 fHistButton(0)
{
   // Constructor.

   MakeTitle("CRESTVetoPaddle");
   fPriority = 0;

   fHFrame0 = new TGHorizontalFrame(this);
   fTubeLabel0 = new TGLabel(fHFrame0, "Tube0:");
   fHFrame0->AddFrame(fTubeLabel0, new TGLayoutHints(kLHintsLeft, 1, 6, 2, 0));
   fBusLabel0 = new TGLabel(fHFrame0, "Bus0:");
   fHFrame0->AddFrame(fBusLabel0, new TGLayoutHints(kLHintsLeft, 1, 6, 2, 0));
   fSedecimLabel0 = new TGLabel(fHFrame0, "STAC0:");
   fHFrame0->AddFrame(fSedecimLabel0, new TGLayoutHints(kLHintsLeft, 1, 6, 2, 0));
   fPinLabel0 = new TGLabel(fHFrame0, "Pin0:");
   fHFrame0->AddFrame(fPinLabel0, new TGLayoutHints(kLHintsLeft, 1, 6, 2, 0));
   AddFrame(fHFrame0, new TGLayoutHints(kLHintsTop, 0, 0, 0, 0));

   fHFrame1 = new TGHorizontalFrame(this);
   fTubeLabel1 = new TGLabel(fHFrame1, "Tube1:");
   fHFrame1->AddFrame(fTubeLabel1, new TGLayoutHints(kLHintsLeft, 1, 6, 2, 0));
   fBusLabel1 = new TGLabel(fHFrame1, "Bus1:");
   fHFrame1->AddFrame(fBusLabel1, new TGLayoutHints(kLHintsLeft, 1, 6, 2, 0));
   fSedecimLabel1 = new TGLabel(fHFrame1, "STAC1:");
   fHFrame1->AddFrame(fSedecimLabel1, new TGLayoutHints(kLHintsLeft, 1, 6, 2, 0));
   fPinLabel1 = new TGLabel(fHFrame1, "Pin1:");
   fHFrame1->AddFrame(fPinLabel1, new TGLayoutHints(kLHintsLeft, 1, 6, 2, 0));
   AddFrame(fHFrame1, new TGLayoutHints(kLHintsTop, 0, 0, 0, 0));

   TGHorizontalFrame* fHFrame2 = new TGHorizontalFrame(this);
	 TGLabel* DrawLabel = new TGLabel(fHFrame2, "Draw ADC:");
	 fHFrame2->AddFrame(DrawLabel, new TGLayoutHints(kLHintsLeft, 1, 6, 2, 0));

   //fHistButton = new TGPictureButton(this, gClient->GetPicture("mb_stop_s.xpm"));
   fHistButton = new TGPictureButton(fHFrame2, gClient->GetPicture("draw_t.xpm"));
   fHistButton->Connect
      ("Clicked()",
       "CRESTVetoPaddleEditor", this, "DoHist()");
   fHistButton->Resize(20,20);
	 fHFrame2->AddFrame(fHistButton, new TGLayoutHints(kLHintsTop, 0, 0, 0, 0));
   AddFrame(fHFrame2, new TGLayoutHints(kLHintsTop, 0, 0, 0, 0));
}

/******************************************************************************/

//______________________________________________________________________________
void CRESTVetoPaddleEditor::SetModel(TObject* obj)
{
   // Set model object.

   fRE = dynamic_cast<CRESTVetoPaddle*>(obj);

	//fBusLabel->UnmapWindow();

	fTubeLabel0->MapWindow();
	fBusLabel0->MapWindow();
	fSedecimLabel0->MapWindow();
	fPinLabel0->MapWindow();
	fTubeLabel1->MapWindow();
	fBusLabel1->MapWindow();
	fSedecimLabel1->MapWindow();
	fPinLabel1->MapWindow();

	/*
   fPreLabel   ->UnmapWindow();
   fRnrSelf    ->UnmapWindow();
   fRnrChildren->UnmapWindow();
   fRnrState   ->UnmapWindow();
   if (fRE->CanEditElement()) {
      fPreLabel->MapWindow();
      if (fRE->SingleRnrState()) {
         fRnrState->SetState(fRE->GetRnrState() ? kButtonDown : kButtonUp);
         fRnrState->MapWindow();
      } else {
         fRnrSelf->SetState(fRE->GetRnrSelf() ? kButtonDown : kButtonUp);
         fRnrChildren->SetState(fRE->GetRnrChildren() ? kButtonDown : kButtonUp);
         fRnrSelf->MapWindow();
         fRnrChildren->MapWindow();
      }
   }

   if (fRE->CanEditMainColor()) {
      fMainColor->SetColor(TColor::Number2Pixel(fRE->GetMainColor()), kFALSE);
      fMainColor->MapWindow();
   } else {
      fMainColor->UnmapWindow();
   }
   if (fRE->CanEditMainTransparency()) {
      fTransparency->SetNumber(fRE->GetMainTransparency());
      fTransparency->MapWindow();
   } else {
      fTransparency->UnmapWindow();
   }
   if (fRE->CanEditMainTrans()) {
      fTrans->SetModel(fRE->PtrMainTrans());
      fTrans->MapWindow();
   } else {
      fTrans->UnmapWindow();
   }
	 */

	 TString content;
	 content = TString::Format("Tube0: %d",fRE->fTubeNum0);
	 fTubeLabel0->SetText(content);
	 content = TString::Format("Bus0: %d",fRE->fBus0);
	 fBusLabel0->SetText(content);
	 content = TString::Format("STAC0: %d",fRE->fSedecim0);
	 fSedecimLabel0->SetText(content);
	 content = TString::Format("Pin0: %d",fRE->fPin0);
	 fPinLabel0->SetText(content);
   fHFrame0->Layout();
	 content = TString::Format("Tube0: %d",fRE->fTubeNum1);
	 fTubeLabel1->SetText(content);
	 content = TString::Format("Bus1: %d",fRE->fBus1);
	 fBusLabel1->SetText(content);
	 content = TString::Format("STAC0: %d",fRE->fSedecim1);
	 fSedecimLabel1->SetText(content);
	 content = TString::Format("Pin1: %d",fRE->fPin1);
	 fPinLabel1->SetText(content);
   fHFrame1->Layout();
}

/******************************************************************************/

void CRESTVetoPaddleEditor::DoHist()
{
   // Slot for drawing this crystal's ADC histogram. TBD: Total, rolling or current event only? For now there is only total.

   cout << "Drawing ADC Histogram for crystal " << fRE->GetElementTitle() << endl;
	 CRESTVetoPaddleView* view = new CRESTVetoPaddleView();
	 view->SetElement(fRE);
	 view->ImportADCHists();
	 view->DrawHist(3);
   Update();
}
