#include <iostream>
#include <stdlib.h>
#include <map>
#include <vector>
#include <TApplication.h>
#include <TRint.h>
#include <TFile.h>
#include <TROOT.h>
#include <string>
#include <TSystem.h>
#include <TGeoManager.h>
#include <TGeoPhysicalNode.h>
#include <TMath.h>
#include <TEveManager.h>
#include <TEveGeoNode.h>
#include <TEveBrowser.h>
#include <TEveScene.h>
#include <TEveTrans.h>
//#include <TEvePointSet.h> 
#include <TEveBoxSet.h> //UC_Events representation in Eve is an array of colored/sized boxes.
#include <TEveRGBAPalette.h> //UC_Events representation in Eve is an array of colored/sized boxes.
#include <TGButton.h>
#include <vector>
#include <RQ_OBJECT.h>
#include <TEveViewer.h>
#include <TGLViewer.h>
#include <TTree.h>
#include <TVector3.h>
#include <TPRegexp.h> //perl regular expression matching for TStrings - cool!

#include "CRESTDetector.h"
#include "CRESTNavHandler.h"
#include "CRESTMultiView.h"
#include "CRESTComponent.h"
#include "CRESTCrystal.h"
#include "CRESTCrystalView.h"
#include "CRESTVetoPaddle.h"
#include "CRESTVetoPaddleView.h"
#include "CRESTHit.h"
#include "UC_Geometry.h" //for TubeNum deliciousness!
#include "UC_Event.h"

#ifndef _CONSTANTS_H_
#include "constants.h"
#endif

#include "CRESTViewer.h"

ClassImp(CRESTViewer);

using namespace std;

TEveBoxSet* fCrystalHitBoxSet;
TEveRGBAPalette* fCrystalPalette;

//______________________________________________________________________________
CRESTViewer::CRESTViewer() //do nothing.
{}

CRESTViewer::CRESTViewer(const TGWindow* p, UInt_t w, UInt_t h) : TGMainFrame(p,w,h) //main constructor.
{
	cout << "Init eve..." << endl;
	init_eve();
	cout << "Init geo..." << endl;
	init_geometry();
	cout << "Init input..." << endl;
	init_input();
	//cout << "Init gui..." << endl;
	init_gui();
	//make_cut_tab(); //when it exists!
	//here is where the first event should be loaded
	//load_event();
}

CRESTViewer::~CRESTViewer()
{}

void CRESTViewer::init_eve() //FIXME
{
	TEveManager::Create(kTRUE,"V"); //F for file browser, I for command line, V for default GL viewer.
	gEve->GetMainWindow()->Resize(1200,800);
	gEve->GetDefaultViewer()->SetName("CREST 3D View");
	gEve->GetBrowser()->SetTabTitle("3D View", TRootBrowser::kRight,0);

	TEveViewer *ev = gEve->GetDefaultViewer();
	TGLViewer *gv = ev->GetGLViewer();
	gv->SetGuideState(TGLUtil::kAxesOrigin,kTRUE,kFALSE,0);
	gv->SetCurrentCamera(TGLViewer::kCameraPerspXOY);


	gEve->Redraw3D(kTRUE);
}

void CRESTViewer::init_gui() 
{
	if(!gEve) {
		cout << "Error in CRESTViewer::init_gui>: No gEve! bad." << endl;
	}
	//draw the multiview tab
  fMultiview = new CRESTMultiView;

	//create a list of teveelements whose name contains "VETO", and include in veto geometry
	TEveElement* geo_top = gEve->GetGlobalScene()->FindChild("TOP_0");
	if(!geo_top) {
		cout << "Error: couldn't find geo top!" << endl;
	} else {
		cout << "Got top element with name " << geo_top->GetElementName() << endl;
	}
	TPRegexp re_veto("VETO");
	TEveElement::List_t veto_elements;
	geo_top->FindChildren(veto_elements,re_veto);
	cout << "Found " << veto_elements.size() << " in the list veto_elements that matches VETO." << endl;
	
	for (TEveElement::List_i p=veto_elements.begin(); p!=veto_elements.end(); ++p)
	{
		fMultiview->ImportGeomVeto((*p));
	}

	//create a list of teveelements whose name contains "BUS", and include in crystal geometry
	//note: this causes crashes. let's try only to include those elements that contain "XTAL"
	/*
	TPRegexp re_bus("BUS");
	TEveElement::List_t crystal_elements;
	geo_top->FindChildren(crystal_elements,re_bus);
	cout << "Found " << crystal_elements.size() << " in the list crystal_elements that matches BUS." << endl;
	
	for (TEveElement::List_i p=crystal_elements.begin(); p!=crystal_elements.end(); ++p)
	{
		//CRESTComponent* comp = dynamic_cast< CRESTComponent*>(*p);
		cout << "Importing element " << (*p)->GetElementName() << " with draw state " << (*p)->GetRnrSelf() << endl;
		fMultiview->ImportGeomCrystal((*p));
	}
	*/
	vector<CRESTComponent*>::iterator compit;
	for (compit=fComponentVector.begin();compit < fComponentVector.end();compit++) {
		TString title = (*compit)->GetElementTitle();
		if(title.Contains("XTAL")) { //then add to the crystal geometry
			cout << "Got XTAL element with name " << (*compit)->GetElementName() << endl;
			fMultiview->ImportGeomCrystal(*compit);
		}
	}
	make_control_tab(); //event stepper etc..

}

void CRESTViewer::init_input()  //Initialize the input method (tree? stream from server?) FIXME
{
  fHandler = new CRESTNavHandler();
	TFile* ipf = new TFile("/Users/jgennaro/Datafiles/Run_00791_fof.root","READ");
	TTree* t = (TTree*) ipf->Get("t");
	fHandler->Init(t);
	cout << "fHandler has current event with hitcount " << fHandler->fCurrentEvent->fHitCount << endl;
}
void CRESTViewer::make_control_tab()  //make a tab for event control
//and tabs for other views?
{
   TEveBrowser* browser = gEve->GetBrowser();
   browser->StartEmbedding(TRootBrowser::kLeft);

   TGMainFrame* frmMain = new TGMainFrame(gClient->GetRoot(), 1000, 600);
   frmMain->SetWindowName("Event Control Window");
   frmMain->SetCleanup(kDeepCleanup);

   TGHorizontalFrame* hf = new TGHorizontalFrame(frmMain);
   {
      TString icondir( Form("%s/icons/", gSystem->Getenv("ROOTSYS")) );
      TGPictureButton* b = 0;
			/*
			cout << "Test navhandler." << endl;
			fHandler->Fwd();
			fHandler->Bck();
			*/

      b = new TGPictureButton(hf, gClient->GetPicture(icondir+"GoBack.gif"));
      hf->AddFrame(b);
      //b->Connect("Clicked()", "CRESTNavHandler", fHandler, "Bck()");
      b->Connect("Clicked()", "CRESTViewer",this,"event_back()");

      b = new TGPictureButton(hf, gClient->GetPicture(icondir+"GoForward.gif"));
      hf->AddFrame(b);
      //b->Connect("Clicked()", "CRESTNavHandler", fHandler, "Fwd()");
      b->Connect("Clicked()", "CRESTViewer",this,"event_forward()");
   }
   frmMain->AddFrame(hf);

   frmMain->MapSubwindows();
   frmMain->Resize();
   frmMain->MapWindow();

   browser->StopEmbedding();
   browser->SetTabTitle("Event Control", 0);
}

void CRESTViewer::load_event() //ask the nav handler to give us an event to load 
{
	if(!fHandler->fCurrentEvent) {
		cout << "Info in <CRESTViewer::load_event>: fHandler passing null event - aborting load event." << endl;
		return;
	}
	cout << "load_event: loading event " << fHandler->fCurrentEventNumber << "." << endl;

	if(fCrystalPalette ==0) { //create the palette
		//gStyle->SetPalette(1,0);
		fCrystalPalette = new TEveRGBAPalette(0,3000);
	}
	if(fCrystalHitBoxSet ==0) { //create the point set
		fCrystalHitBoxSet = new TEveBoxSet();
		fCrystalHitBoxSet->SetOwnIds(kTRUE); //yes, please delete the CRESTHits to avoid memory leak.
		fCrystalHitBoxSet->Reset(TEveBoxSet::kBT_AABox,kTRUE,64); //boolean is 'value is color, 64 is chunk size (???)
		fCrystalHitBoxSet->SetPalette(fCrystalPalette);

		//fCrystalHitBoxSet->SetMainColor(kMagenta);
		//fCrystalHitBoxSet->SetMarkerSize(0.5);
		//fCrystalHitBoxSet->SetMarkerStyle(2);
		//fCrystalHitBoxSet->IncDenyDestroy();
	} else {
		fCrystalHitBoxSet->Reset(TEveBoxSet::kBT_AABox,kTRUE,64); //boolean is 'value is color, 64 is chunk size (???)
	}
	//loop over the hits in the event, pass them to the CRESTHit constructor, then tell the CRESTHit to load itself into the pointset.
	Int_t nhits = fHandler->fCurrentEvent->fHitCount;
	for (Int_t ihit=0;ihit<nhits;ihit++) {
		Int_t tubenum = (Int_t)(fHandler->fCurrentEvent->fTubeNum[ihit]);
		CRESTHit* a_hit = new CRESTHit(fHandler->fCurrentEvent,ihit,fComponentVector[tubenum]); //FIXME: switching from map to vector.
		//the component map entry should pass a pointer to the CRESTComponent that owns this tube.
		//a_hit should initialize itself from this event and index. add it to the box set!
		fCrystalHitBoxSet->AddBox(a_hit->fXLoc, a_hit->fYLoc, a_hit->fZLoc, 3,3,3);
		fCrystalHitBoxSet->DigitValue(a_hit->fADC0);
		cout << "Adding box at " << a_hit->fXLoc << ", " <<  a_hit->fYLoc << ", " <<  a_hit->fZLoc << " with digit value " << a_hit->fADC0 << endl;
	}
	gEve->GetEventScene()->AddElement(fCrystalHitBoxSet);
	gEve->Redraw3D(kTRUE);
}

void CRESTViewer::init_geometry()
{
	fDetector = new CRESTDetector;
	fDetector->Init();
	Int_t gid=0;
	Int_t pid=0;
	gGeoManager->CdTop();
	recursively_descend(0,0,gid,pid);
	fGeom = UC_Geometry::getInstance();
}

void CRESTViewer::recursively_descend(CRESTComponent* pe, Int_t daughter_id,Int_t& global_id, Int_t& paddle_id) {

	TGeoNode   *node = gGeoManager->GetCurrentNode();
	TGeoVolume *vol  = node->GetVolume();
	TString name = TString::Format("%s_%d",vol->GetName(),daughter_id);
	TString title = TString::Format("%s",gGeoManager->GetPath());
	CRESTComponent* this_comp;
	/*if(pe) {
		cout << "The parent named " << pe->GetElementName() << " has CRESTId " << pe->GetId() << "; I have daughter_id = " << daughter_id << " and my CRESTId is " << global_id << endl;
	} else {
		cout << "There was no parent; " << "; I have " << daughter_id << " and my CRESTId is " << global_id << endl;
	}*/

	const Bool_t am_assembly = kTRUE;
	const Bool_t am_not_assembly = kFALSE;
	if(name.Contains("TOP") && !(name.Contains("VETO"))) {
		cout << "Got a TOP!" << " named " << name << endl;
		//make the CRESTComponent for this object. If it is a tube or paddle make it visible; else don't.
		CRESTComponent* comp = new CRESTComponent(name,title,global_id, gGeoManager->GetCurrentMatrix(), (TGeoShape*)(gGeoManager->GetCurrentVolume()->GetShape()->Clone()),am_assembly);
		fComponentVector.push_back(comp);
		//comp->RefMainTrans().SetFrom(*gGeoManager->GetCurrentMatrix());
		//comp->SetShape( (TGeoShape*) gGeoManager->GetCurrentVolume()->GetShape()->Clone());
		//comp->SetRnrSelfChildren(kFALSE,kTRUE); //don't draw assembly volumes, only physical volumes.
		//comp->SetPickable(kFALSE);
		//comp->fIsAssembly=kTRUE;
		gEve->AddGlobalElement(comp); //add as top node of geomeetry scene
		this_comp = comp;
		global_id++;
		//daughters added to this!
	} else {
			if(name.Contains("BUS")) {
				cout << "Got a BUS!" << " named " << name << endl;
				//make the CRESTComponent for this object. If it is a tube or paddle make it visible; else don't.
				CRESTComponent* comp = new CRESTComponent(name,title,global_id, gGeoManager->GetCurrentMatrix(),(TGeoShape*)(gGeoManager->GetCurrentVolume()->GetShape()->Clone()),am_assembly);
				fComponentVector.push_back(comp);
				//comp->RefMainTrans().SetFrom(*gGeoManager->GetCurrentMatrix());
				//comp->SetShape( (TGeoShape*) gGeoManager->GetCurrentVolume()->GetShape()->Clone());
				//comp->SetRnrSelfChildren(kFALSE,kTRUE); //don't draw assembly volumes, only physical volumes.
				//comp->SetPickable(kFALSE);
				//comp->fIsAssembly=kTRUE;
				pe->AddElement(comp); //i'm not top; just add me to the current parent.
				this_comp = comp;
			}
			if(name.Contains("SEDECIM")) {
				cout << "Got a SEDECIM!" << " named " << name << endl;
				CRESTComponent* comp = new CRESTComponent(name,title,global_id, gGeoManager->GetCurrentMatrix(),(TGeoShape*)(gGeoManager->GetCurrentVolume()->GetShape()->Clone()),am_assembly);
				fComponentVector.push_back(comp);
				/*
				comp->RefMainTrans().SetFrom(*gGeoManager->GetCurrentMatrix());
				comp->SetShape( (TGeoShape*) gGeoManager->GetCurrentVolume()->GetShape()->Clone());
				comp->SetRnrSelfChildren(kFALSE,kTRUE); //don't draw assembly volumes, only physical volumes.
				comp->SetPickable(kFALSE);
				comp->fIsAssembly=kTRUE;
				*/
				pe->AddElement(comp); //i'm not top; just add me to the current parent.
				this_comp = comp;
			}
			if(name.Contains("XTAL")) {
				cout << "Got a XTAL!" << " named " << name << endl;
				//CRESTCrystal* crystal = new CRESTCrystal(name,title,global_id, gGeoManager->GetCurrentMatrix(),am_assembly);
				//create a better name using the tube number.
				//crystal->RefMainTrans().SetFrom(*gGeoManager->GetCurrentMatrix());
				//crystal->SetShape( (TGeoShape*) gGeoManager->GetCurrentVolume()->GetShape()->Clone());
				//parse out the bus, sed, pin # from GetElementTitle
				//TString path = crystal->GetElementTitle();
				TString path = title;
				Int_t indBus = path.Index("BUS_");
				Int_t indSed = path.Index("SEDECIM_");
				Int_t indTube = path.Index("XTAL_");
				TString sBus = path(indBus+4,indSed - indBus-5);
				TString sSed = path(indSed+8,indTube - indSed-9);
				TString sTube = path(indTube+5,path.Length());
				Int_t nBus = sBus.Atoi();
				Int_t nSed = sSed.Atoi();
				Int_t nPin = sTube.Atoi();
				cout <<"\t\t Got bus, sed pin = " << nBus << ", " << nSed<< ", " << nPin << "!" << endl;
				UC_Geometry* geo = UC_Geometry::getInstance(UC_Geometry::POLAR);
				Int_t tubenum = geo->getTubeID(false,nBus+1,nSed+1,nPin);
				TString newname = TString::Format("XTAL_%d",tubenum);
				CRESTCrystal* crystal = new CRESTCrystal(newname,title,global_id, gGeoManager->GetCurrentMatrix(),(TGeoShape*)(gGeoManager->GetCurrentVolume()->GetShape()->Clone()), am_not_assembly);
				crystal->fTubeNum = tubenum;
				crystal->fBus = nBus;
				crystal->fSedecim = nSed;
				crystal->fPin = nPin;
				cout <<"\t\t Got bus, sed pin = " << crystal->fBus << ", " << crystal->fSedecim << ", " << crystal->fPin << "!" << endl;
				//add this to the map: key value fTubeNum, element value fCRESTId
				fTubeIdMap.insert(pair<Int_t,Int_t>(crystal->fTubeNum,crystal->fCRESTId));
				fComponentVector.push_back(crystal); //it's now the fCRESTId'th entry in this vector; hence need the map
				cout << "\t setting crystal " << crystal->fTubeNum << "'s position to " << crystal->fXLoc << " : " << crystal->fYLoc << endl;
				pe->AddElement(crystal); //i'm not top; just add me to the current parent.
				this_comp = crystal;
			}
			if(name.Contains("VETO")) {
				cout << "Got a VETO!" << " named " << name << endl;
				CRESTComponent* comp = new CRESTComponent(name,title,global_id, gGeoManager->GetCurrentMatrix(),(TGeoShape*)(gGeoManager->GetCurrentVolume()->GetShape()->Clone()),am_assembly);
				fComponentVector.push_back(comp);
				//comp->RefMainTrans().SetFrom(*gGeoManager->GetCurrentMatrix());
				//comp->SetShape( (TGeoShape*) gGeoManager->GetCurrentVolume()->GetShape()->Clone());
				//comp->SetRnrSelfChildren(kFALSE,kTRUE); //don't draw assembly volumes, only physical volumes.
				//comp->SetPickable(kFALSE);
				//comp->fIsAssembly=kTRUE;
				pe->AddElement(comp); //i'm not top; just add me to the current parent.
				this_comp=comp;
			}
			if(name.Contains("PADDLE")) {
				cout << "Got a PADDLE!" << " named " << name << endl;
				CRESTVetoPaddle* paddle = new CRESTVetoPaddle(name,title,global_id, gGeoManager->GetCurrentMatrix(),(TGeoShape*)(gGeoManager->GetCurrentVolume()->GetShape()->Clone()),am_not_assembly);
				fComponentVector.push_back(paddle);
				//paddle->RefMainTrans().SetFrom(*gGeoManager->GetCurrentMatrix());
				//paddle->SetShape( (TGeoShape*) gGeoManager->GetCurrentVolume()->GetShape()->Clone());
				//the index of this paddle is paddle_id, from 0-26 
				//the tubes that belong to this paddle, from 0-53, are paddle_id*2 and paddle_id*2+1
				//the addresses are Addresses[paddle_id*2] and Addresses[paddle_id*2+1]
				paddle->fBus0 = Addresses[paddle_id*2][0]-1;
				paddle->fSedecim0 = Addresses[paddle_id*2][1]-1;
				paddle->fPin0 = Addresses[paddle_id*2][2];
				paddle->fBus1 = Addresses[paddle_id*2+1][0]-1;
				paddle->fSedecim1 = Addresses[paddle_id*2+1][1]-1;
				paddle->fPin1 = Addresses[paddle_id*2+1][2];

				UC_Geometry* geo = UC_Geometry::getInstance(UC_Geometry::POLAR);
				paddle->fTubeNum0 = geo->getTubeID(true,paddle->fBus0+1,paddle->fSedecim0+1,paddle->fPin0);
				cout << "geo returns tube " << paddle->fTubeNum0 << " from " << paddle->fBus0+1 << ", " << paddle->fSedecim0+1 << ", " << paddle->fPin0 << endl;
				paddle->fTubeNum1 = geo->getTubeID(true,paddle->fBus1+1,paddle->fSedecim1+1,paddle->fPin1);
				cout << "geo returns tube " << paddle->fTubeNum1 << " from " << paddle->fBus1+1 << ", " << paddle->fSedecim1+1 << ", " << paddle->fPin1 << endl;
				//add this to the map: key value fTubeNum, element value fCRESTId
				fTubeIdMap.insert(pair<Int_t,Int_t>(paddle->fTubeNum0,paddle->fCRESTId));
				fTubeIdMap.insert(pair<Int_t,Int_t>(paddle->fTubeNum1,paddle->fCRESTId));
				//TEveTrans ttrans = paddle->RefMainTrans();
				//TVector3 tvec = ttrans.GetPos();
				//paddle->fXLoc = tvec.X();
				//paddle->fYLoc = tvec.Y();
				//paddle->fZLoc = tvec.Z();
				if(title.Contains("X") || title.Contains("BOT")) {
					paddle->fLongDim = 1; //1 -> long in Y
				} else if(title.Contains("Y") || title.Contains("TOP")) {
					paddle->fLongDim = 0; //0 -> long in X
				}
				//paddle->fLongDimLength = tvec(paddle->fLongDim);
				paddle->fLongDimLength = 0;
				
				//paddle->SetRnrSelf(kTRUE); //yes, please draw this as it's not an assembly but a physical volume.
				//paddle->SetPickable(kTRUE);
				//paddle->SetMainColor(kGreen);
				//paddle->fIsAssembly=kFALSE;
				paddle->SetMainTransparency(VETO_TRANS);
				pe->AddElement(paddle); //i'm not top; just add me to the current parent.
				this_comp=paddle;
				paddle_id++;
			}
			global_id++;
		}
		cout << "About to descend through geo with this_comp = " << this_comp->GetElementName() << endl;
	Int_t nd = vol->GetNdaughters();
	for (int i=0;i<nd;i++) {
		gGeoManager->CdDown(i);
		recursively_descend(this_comp,i,global_id,paddle_id);
		gGeoManager->CdUp();
	}
}

void CRESTViewer::event_back() //receives clicked signal of event back button
{
	cout << "Event backing! " << endl;
}

void CRESTViewer::event_forward()//receives clicked signal of event forward button
{
	cout << "Event forwardering!! " << endl;
}

void CRESTViewer::clear_event() //FIXME
{
	cout << "Clearing event!" << endl;
}
