///////////////////////////////////////////////////////////////////////////////
// $Id$
//
// Filename : UC_Event.cpp
// Created  : Wed Mar 26 06:54:24 2008 by S.P. Wakely
///////////////////////////////////////////////////////////////////////////////

#include <TRandom.h>
#include <algorithm> 
#include <sstream>  
#include <iomanip>
#include <iostream>

#include <map>

#include "UC_Event.h"

#if !defined(__CINT__)
  ClassImp(UC_Event)
#endif


//void UC_Event::plotMe(){
//  fP = new UC_GPlotter(UC_Geometry::POLAR);
//  fP->drawADC(this);
//} 

static const unsigned MAX_EVENT_HITS=4096;

///////////////////////////////////////////////////////////////////////////////
/// Boring Constructor
///
///////////////////////////////////////////////////////////////////////////////
UC_Event::UC_Event() 
{  
  fHitCount = 0;
  fWordCount= 0;
  fWordCountSaved = 0;
  fT        = 0;
  fCorLevel = 0;
  fIsCal    = 0;
  fWatchdogCount = 0;

  fTubeNum  = 0;
  fADC0     = 0;
  fADC1     = 0;
  fTDC      = 0;
  fTHold    = 0;
  fBusId    = 0;
  fSTACId   = 0;
  fPMTId    = 0;
  fTubeNum  = new int16_t[MAX_EVENT_HITS];
  fTubeHits = new int16_t[MAX_EVENT_HITS];
  fADC0     = new float[MAX_EVENT_HITS];
  fADC1     = new float[MAX_EVENT_HITS];
  fTDC      = new float[MAX_EVENT_HITS];
  fTHold    = new float[MAX_EVENT_HITS];
  fWithADC  = new int8_t[MAX_EVENT_HITS];
  fIsVeto   = new int8_t[MAX_EVENT_HITS];
  fBusId    = new int16_t[MAX_EVENT_HITS];
  fSTACId   = new int16_t[MAX_EVENT_HITS];
  fPMTId    = new int16_t[MAX_EVENT_HITS];

  fErrCount = 0;
  fDeadTime      = 0;
  fFreezeTick    = 0;
  fEventCount	 = 0;
  fEventId	 = 0;
  fT0	    = 0;
}

///////////////////////////////////////////////////////////////////////////////
/// Boring Deconstructor
///
///////////////////////////////////////////////////////////////////////////////
UC_Event::~UC_Event()
{
  //don't want any memory leaks!
  //clearData();
  
  delete fTubeNum;
  delete fTubeHits;
  delete fADC0;
  delete fADC1;
  delete fTDC;
  delete fTHold;
  delete fWithADC;
  delete fIsVeto;
 
  delete fBusId;
  delete fSTACId;
  delete fPMTId;
 
}


///////////////////////////////////////////////////////////////////////////////
/// Maybe a nice way to get some data out of an event, eh
///
///////////////////////////////////////////////////////////////////////////////
vector<tubePair> UC_Event::getADC(bool isADC0)
{
  vector<tubePair> out(fHitCount);
  for(int n=0;n<fHitCount;n++){
    out[n].tnum = fTubeNum[n];
    if( isADC0 )
      out[n].tval = fADC0[n];
    else
      out[n].tval = fADC1[n];
  }
  return out;
}

///////////////////////////////////////////////////////////////////////////////
/// Maybe a nice way to get some data out of an event, eh
///
///////////////////////////////////////////////////////////////////////////////
vector<tubePair> UC_Event::getTDC()
{
  vector<tubePair> out(fHitCount);
  for(int n=0;n<fHitCount;n++){
    out[n].tnum = fTubeNum[n];
    out[n].tval = fTDC[n];
  }
  return out;
}

///////////////////////////////////////////////////////////////////////////////
/// Prep the event for new data
///
///////////////////////////////////////////////////////////////////////////////
void UC_Event::clearData()
{
  fHitCount = 0;
  fWordCount = 0;
  fWordCountSaved = 0;
  fT        = 0;
  fCorLevel = 0;
  fIsCal    = 0;
  fErrCount = 0;
 
//  for(int n=0;n<MAX_EVENT_HITS;n++) fTubeHits[n] = 0;
//  for(int n=0;n<MAX_EVENT_HITS;n++) fTubeNum[n] = 0;

  //fixme: need to zero out data better!
}

///////////////////////////////////////////////////////////////////////////////
/// fixme:
/// Space the arrays appropriately.  Not sure I really like this.
/// if I pre-allocate and properly set fHitCount, won't ROOT do
/// the size properly, but also quickly?
///
///////////////////////////////////////////////////////////////////////////////
void UC_Event::setDataSize(uint16_t size)
{
  fHitCount = size;
  
  if(fHitCount >= MAX_EVENT_HITS){
    cerr << "Here comes a set fault!" << endl;
  }
  
#if 0
  // no memory leaks, please.
  try{
    if(fTubeNum) delete fTubeNum;
    if(fADC0)    delete fADC0;
    if(fADC1)    delete fADC1;
    if(fTDC)     delete fTDC;
    if(fTHold)   delete fTHold;
    
    fTubeNum  = new int16_t[fHitCount];
    fADC0     = new float[fHitCount];
    fADC1     = new float[fHitCount];
    fTDC      = new float[fHitCount];
    fTHold    = new float[fHitCount];
  }
  catch (exception& e)
  {
    cout << e.what() << endl;
  }
#endif 
  
}

///////////////////////////////////////////////////////////////////////////////
/// Test fn for generating fake data!
///
///////////////////////////////////////////////////////////////////////////////
void UC_Event::fillRandom(double inT)
{
  fT          = inT;
  fEventId   = int(inT);
  
  setDataSize(1 + gRandom->Integer(30));
  
  // A quick method of generating 30 unique randoms
  // see svn for more elegant but slower way
  map<int,int> m;
  int uniqCount = 0;
  while(uniqCount < fHitCount){
    int x = gRandom->Integer(1024);
    if(m.find(x)==m.end()){
      m[x] = x;
      uniqCount++;
    }
  }
  
  map<int,int>::iterator iter = m.begin();
  for(int n=0;n<fHitCount;n++){
    fTubeNum[n] = iter->first;
    iter++;
    fADC0[n]   = gRandom->Uniform(1024)+10;
    fADC1[n]   = gRandom->Uniform(1024)+10;
    fTDC[n]    = (int)gRandom->Uniform(60);
    fTHold[n]  = gRandom->Uniform(30);    
  }  
  
}

///////////////////////////////////////////////////////////////////////////////
/// Dump some info to a string love to inform the lusers
///
///////////////////////////////////////////////////////////////////////////////
string UC_Event::dumpInfo(bool printOut){
  
  ostringstream out;
  
  out << "  -== Event Dump ==-" << endl;
  out << "  Evt Time  :" << fT << endl;
  out << "  Evt Id   :" << fEventId << endl;
  out << "  Hit Count :" << fHitCount << endl;

  int max = fHitCount;
  if(max > 8) max = 8;
  out << "  UC_GTube: ";
  for(int n=0;n<max;n++) out << left << setw(8) << fTubeNum[n];
  out << endl << "  ADC:  ";
  for(int n=0;n<max;n++) out << left << setw(8) << setprecision(4) << fADC0[n];
  out << endl << "  TDC:  ";
  for(int n=0;n<max;n++) out << left << setw(8) << setprecision(4) << fTDC[n];
  out << endl;
  
  if(printOut) cout << out.str();
  return out.str();
  
}

