///////////////////////////////////////////////////////////////////////////////
// $Id$
//
// Filename : UC_Geometry.cpp
// Created  : Wed Mar 26 08:37:14 2008 by S.P. Wakely
///////////////////////////////////////////////////////////////////////////////

#include "UC_Geometry.h"

#include <iostream>
#include <fstream>

using namespace std;

// Set up the statics
UC_Geometry *UC_Geometry::theGeometry = 0;
typedef pair<UC_Geometry::Config,string> t_p; typedef map<UC_Geometry::Config,string> t_m;
t_p mi[] = {t_p(UC_Geometry::CONUS,string("CONUS")),t_p(UC_Geometry::POLAR,string("POLAR")),t_p(UC_Geometry::TEST,string("TEST"))};
t_m UC_Geometry::fConfigNames = t_m(mi, mi+sizeof(mi)/sizeof(mi[0]));

///////////////////////////////////////////////////////////////////////////////
/// This is a private constructor
/// I think we want to do this class as a singleton with Lazy initialization
//
///////////////////////////////////////////////////////////////////////////////
UC_Geometry::UC_Geometry(Config inConf)
{  
  
  fConfig = inConf;
  cout << "UC_Geometry Instantiated as: " << fConfigNames[fConfig] << endl;
  
  if(fConfig == POLAR){
    fTubeCount = 1024 + 54;
    fXmin      = -140.0;
    fYmin      = -140.0;
    fXmax      = 140.0;
    fYmax      = 140.0;
    fRmax      = 3.0;
    fXRowCount = 32;
    fYRowCount = 32;
    
    int cnt = 0;
    int paddle_cnt = 0;

    //crystal
    for(int n=0;n<fYRowCount;n++){
      for(int m=0;m<fXRowCount;m++){
        //fXPos[cnt] = fRmax + double(m)*fRmax*2 + 2*int(m/4);
        //fYPos[cnt] = fRmax + double(n)*fRmax*2 + 2*int(n/4);
        fXPos[cnt] = fRmax + double(m)*fRmax*2 - fXRowCount*2*fRmax/2.0;
        fYPos[cnt] = fRmax + double(n)*fRmax*2 - fXRowCount*2*fRmax/2.0;
        fFake[cnt] = false;

    //addresses
        fTubeID[cnt] = cnt;
        fBUSAddr[cnt] = int(cnt/128)+1; 
        fSTACAddr[cnt] = int(int(cnt/4)%8)+1;
        int pmtIdInBus = cnt - 128*(fBUSAddr[cnt]-1);
        int rowInSTAC = int(pmtIdInBus/32);
        fChannelAddr[cnt] = (pmtIdInBus-4*(fSTACAddr[cnt]-1) - rowInSTAC*32)*4 + rowInSTAC;
        fPaddleID[cnt] = -1;

        cnt++;
      }
    }

   //nahee : for now, hard coded (from Jan. 11 2011 document)
	/*
    int veto_bus[54] = { 4, 3, 3, 4, 3,    4, 3, 4, 2, 1,
			 2, 1, 2, 1, 2,    1, 3, 2, 3, 2,
			 3, 2, 3, 2, 4,    2, 4, 1, 4, 1,
			 4, 1, 4, 1, 3,    2, 4, 1, 3, 4,
			 2, 1, 3, 2, 4,    1, 3, 3, 4, 4,
			 2, 1, 1, 1 };

    int veto_stac[54] = {2, 2, 2, 2, 2,    2, 2, 2, 2, 2,
			 2, 2, 2, 2, 1,    2, 1, 1, 1, 1,
			 1, 1, 1, 1, 2,    2, 1, 1, 1, 1,
			 1, 1, 1, 1, 2,    2, 2, 2, 1, 2,
			 2, 2, 1, 1, 2,    2, 2, 2, 1, 1,
			 1, 1, 1, 1};

    int veto_channel[54] = {3, 4, 2, 7, 0,    1, 1, 2, 1, 0,
			    2, 1, 3, 2, 7,    7, 0, 0, 1, 1,
			    2, 2, 5, 3, 0,    0, 0, 0, 1, 1,
			    2, 2, 3, 3, 5,    6, 5, 4, 7, 6,
			    5, 5, 6, 5, 4,    6, 7, 6, 4, 6,
			    6, 5, 7, 6};
	*/
	//Joe: There are a couple changes to bottom paddles 4 and 5 (as of 5/16/2011, taken from Mike Lang's veto tech document)
	/*
    int veto_bus[54] = { 4, 3, 3, 4, 3,    4, 3, 4, 2, 1,
		         2, 1, 2, 1, 2,    1, 3, 2, 3, 2,
			 3, 2, 3, 2, 4,    2, 4, 1, 4, 1,
			 4, 1, 4, 1, 3,    2, 4, 1, 3, 4,
			 2, 1, 3, 2, 4,    1, 3, 3, 4, 4,
			 2, 1, 1, 1 };

    int veto_stac[54] = {2, 2, 2, 2, 2,    2, 2, 2, 2, 2,
			 2, 2, 2, 2, 1,    2, 1, 1, 1, 1,
			 1, 1, 1, 1, 1,    2, 1, 1, 1, 1,
			 1, 1, 1, 1, 2,    2, 2, 2, 1, 2,
			 2, 2, 1, 1, 2,    2, 2, 2, 1, 1,
			 1, 1, 1, 1};

    int veto_channel[54] = {3, 4, 2, 7, 0,    1, 1, 2, 1, 0,
			    2, 1, 3, 2, 7,    7, 0, 0, 1, 1,
			    2, 2, 4, 3, 5,    5, 0, 4, 1, 1,
			    2, 2, 3, 3, 5,    6, 9, 8, 7, 6,
			    4, 5, 6, 5, 4,    6, 7, 6, 4, 6,
			    6, 5, 7, 6};
		*/
		//Joe: Even more changes - this is taken from overlord_procs.h on 6/6/2011.
    int veto_bus[54] = { 3, 4, 3, 4, 3,    4, 3, 4, 2, 1,
		         2, 1, 2, 1, 2,    1, 3, 2, 3, 2,
			 3, 2, 3, 2, 4,    2, 4, 1, 4, 1,
			 4, 1, 4, 1, 3,    2, 4, 1, 3, 4,
			 2, 1, 3, 2, 4,    1, 3, 3, 4, 4,
			 2, 1, 1, 1 };

    int veto_stac[54] = {2, 2, 2, 2, 2,    2, 2, 2, 2, 2,
			 2, 2, 2, 2, 1,    2, 1, 1, 1, 1,
			 1, 1, 1, 1, 1,    2, 1, 1, 1, 1,
			 1, 1, 1, 1, 2,    2, 2, 2, 1, 2,
			 2, 2, 1, 1, 2,    2, 2, 2, 1, 1,
			 1, 1, 1, 1};

    int veto_channel[54] = {4, 3, 2, 7, 0,    1, 1, 2, 1, 0,
			    2, 1, 3, 2, 7,    7, 0, 0, 1, 1,
			    2, 2, 4, 3, 5,    5, 0, 4, 1, 1,
			    2, 2, 3, 3, 5,    7, 9, 8, 7, 6,
			    4, 5, 6, 5, 4,    6, 7, 6, 4, 6,
			    6, 5, 7, 6};


    // top veto
    for(int i=0; i<8; i++){
      for(int j=0; j<2; j++){
        fXPos[cnt] =  103 * (-1 + 2*j ) ;
        fYPos[cnt] = fRmax*4 + i * fRmax*8 - fXRowCount*2*fRmax/2.0;
        fFake[cnt] = false;

        fPaddleXPos[paddle_cnt][j] =  100 * (-1 + 2*j ) ;
        fPaddleYPos[paddle_cnt][j] = (i+j) * fRmax*8 - fXRowCount*2*fRmax/2.0;

        fTubeID[cnt] = paddle_cnt*2 + j;
        fBUSAddr[cnt] = veto_bus[paddle_cnt*2 + j];
        fSTACAddr[cnt] = veto_stac[paddle_cnt*2 + j];
        fChannelAddr[cnt] = veto_channel[paddle_cnt*2 + j];
        fPaddleID[cnt] = paddle_cnt;
        fTubeGUIIDForVeto[fBUSAddr[cnt]-1][fSTACAddr[cnt]-1][fChannelAddr[cnt]] = cnt;
        cnt ++;
      }
      paddle_cnt ++;
    }

    //bottom veto
    for(int i=0; i<9; i++){
      for(int j=0; j<2; j++){
        if( i == 5){
          fXPos[cnt] = 0;
          fYPos[cnt] =  109 * (-1 + 2*j ) ;

          fPaddleXPos[paddle_cnt][j] = fRmax*4*(-1 + 2*j );
          fPaddleYPos[paddle_cnt][j] =  106 * (-1 + 2*j ) ;

        } else {
          if( i<5 ){
            fXPos[cnt] = fRmax*4 + i * fRmax*8 - fXRowCount*2*fRmax/2.0;
            fPaddleXPos[paddle_cnt][j] = (i+j) * fRmax*8 - fXRowCount*2*fRmax/2.0;
          } else {
            fXPos[cnt] = fRmax*4 + (i-1) * fRmax*8 - fXRowCount*2*fRmax/2.0;
            fPaddleXPos[paddle_cnt][j] = (i+j-1) * fRmax*8 - fXRowCount*2*fRmax/2.0;
          }

          fYPos[cnt] =  103 * (-1 + 2*j ) ;
          fPaddleYPos[paddle_cnt][j] =  100 * (-1 + 2*j ) ;
        }
        fFake[cnt] = false;

        fTubeID[cnt] = paddle_cnt*2 + j;
        fBUSAddr[cnt] = veto_bus[paddle_cnt*2 + j];
        fSTACAddr[cnt] = veto_stac[paddle_cnt*2 + j];
        fChannelAddr[cnt] = veto_channel[paddle_cnt*2 + j];
        fPaddleID[cnt] = paddle_cnt;
        fTubeGUIIDForVeto[fBUSAddr[cnt]-1][fSTACAddr[cnt]-1][fChannelAddr[cnt]] = cnt;

        cnt ++;
      }
      paddle_cnt ++;
    }
 
    //side veto
    double loc[3][2] = {{112, 100}, {120,80}, {0,0}};//side, inside1, inside2
    for(int i=0; i<4; i++){
      for(int j=0; j<2; j++){
        if( i <2 ){ //-X/+X
          fXPos[cnt] = loc[0][0] *(-1 + 2* i); 
          fYPos[cnt] = loc[0][1] *(-1+ 2*j);
        } else { //-Y/+Y
          fXPos[cnt] = loc[0][1] * (-1+2*j);
          fYPos[cnt] = (loc[0][0] + 4) * (-5 + 2*i);
        }
        fFake[cnt] = false;

        fPaddleXPos[paddle_cnt][j] = fXPos[cnt] - fRmax * (-1 + 2*j );
        fPaddleYPos[paddle_cnt][j] = fYPos[cnt] - fRmax * (-1 + 2*j );

        fTubeID[cnt] = paddle_cnt*2 + j;
        fBUSAddr[cnt] = veto_bus[paddle_cnt*2 + j];
        fSTACAddr[cnt] = veto_stac[paddle_cnt*2 + j];
        fChannelAddr[cnt] = veto_channel[paddle_cnt*2 + j];
        fPaddleID[cnt] = paddle_cnt;
        fTubeGUIIDForVeto[fBUSAddr[cnt]-1][fSTACAddr[cnt]-1][fChannelAddr[cnt]] = cnt;
        cnt ++;
      }
      paddle_cnt ++;
    }

    //inside slanted veto
    //-X/+X 
    for(int i=0; i<2; i++){
      for(int j=0; j<2; j++){
        fXPos[cnt] = loc[1][0] *(-1 + 2* i);
        fYPos[cnt] = loc[1][1] *(-1+ 2*j);
        fFake[cnt] = false;

        fPaddleXPos[paddle_cnt][j] = fXPos[cnt] - fRmax * (-1 + 2*j );
        fPaddleYPos[paddle_cnt][j] = fYPos[cnt] - fRmax * (-1 + 2*j );

        fTubeID[cnt] = paddle_cnt*2 + j;
        fBUSAddr[cnt] = veto_bus[paddle_cnt*2 + j];
        fSTACAddr[cnt] = veto_stac[paddle_cnt*2 + j];
        fChannelAddr[cnt] = veto_channel[paddle_cnt*2 + j];
        fPaddleID[cnt] = paddle_cnt;
        fTubeGUIIDForVeto[fBUSAddr[cnt]-1][fSTACAddr[cnt]-1][fChannelAddr[cnt]] = cnt;

        cnt ++;
      }
      paddle_cnt ++;
    }

    //-Y/+Y 
    for(int i=0; i<2; i++){
      for(int j=0; j<2; j++){
        for(int k=0; k<2; k++){
          if( j == 0 && k == 1 )
            fXPos[cnt] = -10;
          else if ( j == 1 && k == 0)
            fXPos[cnt] = 10;
          else
            fXPos[cnt] = loc[1][1] * (-1+2*j);
          fYPos[cnt] = (loc[1][0] + 4) * (-1 + 2*i);
          fFake[cnt] = false;

          fPaddleXPos[paddle_cnt][k] = fXPos[cnt] - fRmax * (-1 + 2*k );
          fPaddleYPos[paddle_cnt][k] = fYPos[cnt] - fRmax * (-1 + 2*k );

          fTubeID[cnt] = paddle_cnt*2 + k;
          fBUSAddr[cnt] = veto_bus[paddle_cnt*2 + k];
          fSTACAddr[cnt] = veto_stac[paddle_cnt*2 + k];
          fChannelAddr[cnt] = veto_channel[paddle_cnt*2 + k];
          fPaddleID[cnt] = paddle_cnt;
          fTubeGUIIDForVeto[fBUSAddr[cnt]-1][fSTACAddr[cnt]-1][fChannelAddr[cnt]] = cnt;

//        cout << i <<" " << j <<" " << k <<" " << fXPos[cnt] <<" " << fYPos[cnt]  <<" " << fPaddleXPos[paddle_cnt][k] <<" " << fPaddleYPos[paddle_cnt][k]<<endl;
          cnt ++;
        }
        paddle_cnt ++;
      }
    }


    //total number of tested paddle
    fPaddleCount = paddle_cnt;
    //for(int i=1024; i<1024+54; i++)
    //  cout << i <<" " << fPaddleID[i] <<" " <<  fTubeID[i] << " " << fBUSAddr[i] 
     //     <<" " << fSTACAddr[i]  <<" " << fChannelAddr[i] << " " << fTubeGUIIDForVeto[fBUSAddr[i]-1][fSTACAddr[i]-1][fChannelAddr[i]] << endl;

  } else if(fConfig == CONUS){
    fTubeCount = 128 + 32 ; //64 : crystal, 9 : veto
    fXmin      = 0.0;
    fYmin      = 0.0;
    fXmax      = 200.0;
    fYmax      = 200.0;
    fRmax      = 3.0;
    fXRowCount = 32;
    fYRowCount = 4;
    
    int cnt = 0;
    int add_stac = 0;
    for(int n=0;n<fYRowCount;n++){
      for(int m=0;m<fXRowCount;m++){
//        fXPos[cnt] = 63.0 + double(m)*fRmax*2;
        fXPos[cnt] = 4+fRmax + double(m)*fRmax*2;
        fYPos[cnt] = 91.0 + double(n)*fRmax*2;
        add_stac = int(m/4) + 1;
        if(add_stac == 2 || add_stac == 4 || add_stac == 5 || add_stac == 7)
          fFake[cnt] = true;
        else
          fFake[cnt] = false;
        cnt++;
      }
    }

    for(int n=128; n<160; n++){
      fXPos[cnt] = -999;
      fYPos[cnt] = -999;
      fFake[cnt] = false;
      cnt ++;
    }
    // -x side-------
    // -y side veto
    fXPos[128] = 4+fRmax; // + -1*fRmax*2; 
    fYPos[128] = 91.0 + -2*fRmax*2;
    // top veto
    fXPos[129] = 4+fRmax;
    fYPos[129] = 91.0 + (double(fYRowCount)+5)*fRmax*2;
    // +y side veto
    fXPos[130] = 4+fRmax; 
    fYPos[130] = 91.0 + (double(fYRowCount)+1)*fRmax*2;
    // bottom veto
    fXPos[131] = 4+fRmax; 
    fYPos[131] = 91.0 + -5*fRmax*2;
    // +x side-------
    // -y side veto
    fXPos[144] = 4+fRmax + (double(fXRowCount)-1)*fRmax*2; 
    fYPos[144] = 91.0 + -2*fRmax*2;
    // top veto
    fXPos[145] = 4+fRmax + (double(fXRowCount)-1)*fRmax*2; 
    fYPos[145] = 91.0 + (double(fYRowCount)+5)*fRmax*2;
    // +y side veto
    fXPos[146] = 4+fRmax + (double(fXRowCount)-1)*fRmax*2; 
    fYPos[146] = 91.0 + (double(fYRowCount)+1)*fRmax*2;
    // bottom veto
    fXPos[147] = 4+fRmax + (double(fXRowCount)-1)*fRmax*2; 
    fYPos[147] = 91.0 + -5*fRmax*2;
    // gap strap
    fXPos[148] = 4+fRmax + (double(fXRowCount)-1)*fRmax*2;
    fYPos[148] = 91.0 + (double(fYRowCount)+7)*fRmax*2;

  }//else if(fConfig == CONUS) 
  else if (fConfig == TEST) {
    
    /*
    fTubeCount = 16;
    fXmax      = 120.0;
    fYmax      = 120.0;
    fRmax      = 10.0;
    fXRowCount = 4;
    fYRowCount = 4;
    int cnt = 0;
    for(int n=0;n<fYRowCount;n++){
      for(int m=0;m<fXRowCount;m++){
        fXPos[cnt] = 30.0 + double(m)*fRmax*2;
        fYPos[cnt] = 30.0 + double(n)*fRmax*2;
        cnt++;
      }
    }
    */
   // nahee : New module numbering Jan 29 '10
    fTubeCount = 18;
    fXmin      = 0.0;
    fYmin      = 0.0;
    fXmax      = 160.0;
    fYmax      = 160.0;
    fRmax      = 10.0;
    fXRowCount = 4;
    fYRowCount = 4;
    int cnt = 0;
    for(int m=0;m<fXRowCount;m++){
      for(int n=0;n<fYRowCount;n++){
        fXPos[cnt] = 50.0 + double(m)*fRmax*2;
        fYPos[cnt] = 30.0 + double(n)*fRmax*2;
        fFake[cnt] = false;
        cnt++;
      }
    }
    fXPos[16] = 50.0; fYPos[16] = 130.0;
    fXPos[17] = 70.0; fYPos[17] = 130.0;
     
    fTubeCount = 19;
    fXmax      = 120.0;
    fYmax      = 130.0;
    fRmax      = 10.0;
    fXRowCount = 4;
    fYRowCount = 4;
    cnt = 0;
    for(int m=0;m<fXRowCount;m++){
      for(int n=0;n<fYRowCount;n++){
        fXPos[cnt] = 30.0 + double(m)*fRmax*2;
        fYPos[cnt] = 30.0 + double(n)*fRmax*2;
        cnt++;
      }
    }

    fXPos[16] = 40.0; fYPos[16] = 115.0;
    fXPos[17] = 60.0; fYPos[17] = 115.0;
    fXPos[18] = 80.0; fYPos[18] = 115.0;
    
    
    
  } //else if (fConfig == TEST) {
    
  
}

///////////////////////////////////////////////////////////////////////////////
///
///////////////////////////////////////////////////////////////////////////////
UC_Geometry::~UC_Geometry()
{
  
}

///////////////////////////////////////////////////////////////////////////////
/// Returns a point to the geom class with the current 
///////////////////////////////////////////////////////////////////////////////
UC_Geometry* UC_Geometry::getInstance()
{  
  // We default to POLAR for now!
  if(theGeometry == 0){
    theGeometry = new UC_Geometry(POLAR);
  } 
  return theGeometry;
}

///////////////////////////////////////////////////////////////////////////////
/// Returns a point to the geom class with the proper config
/// ...could actually allow an instance of each type here, but
/// there seems little use for that.
///////////////////////////////////////////////////////////////////////////////
UC_Geometry* UC_Geometry::getInstance(Config inConfig)
{  
  if(theGeometry == 0){
    theGeometry = new UC_Geometry(inConfig);
  } 
  return theGeometry;
}

///////////////////////////////////////////////////////////////////////////////
/// Generate a tube ID based on the addressing
///  
///
///////////////////////////////////////////////////////////////////////////////

// this really should only be calculated once, eh?
int UC_Geometry::getTubeID(bool is_veto, int addBus, int addSTAC, int addTube)
{
// nahee : New module numbering Jan 29 '10
  int rowInSTAC;
  int posInRow;
  int val;

  //Test config ignores CRESTbus and STAC Address
  if(fConfig == TEST){
    val = addTube;

  } else if(fConfig == CONUS ){
    rowInSTAC = int(addTube/4);
    posInRow  = addTube - 4*rowInSTAC;
		/*
    if( addSTAC == 7)
      val = (addBus-1)*128 + (rowInSTAC)*32 + (addSTAC)*4 + posInRow;
    if( addBus == 4)
      val = 128 + addTube;
    if( addBus == 3)
      val = 128 + 16 + addTube;
		*/
    if( addBus == 3 && addSTAC == 7)
      val = (rowInSTAC)*32 + (addSTAC)*4 + posInRow;
    if( addBus == 3 && addSTAC != 7)
      val = (rowInSTAC)*32 + (addSTAC-1)*4 + posInRow;
    if( addBus == 4)
      val = 128 + addTube;
    if( addBus == 1)
      val = 128 + 16 + addTube;

  } else { //polar
// nahee : include veto bit information May 4 '10
    if( !is_veto ){ //crystal
      rowInSTAC = int(addTube%4);
      posInRow  = int(addTube/4);
      val = (addBus-1)*128 + (rowInSTAC)*32 + (addSTAC-1)*4 + posInRow;

      if(val > 1077 || val < 0){
        cerr << "Bad Dog in UC_Geometry::getTubeID : "
              << addBus <<" " << addSTAC << " " <<addTube <<" " << val <<endl;
        cout <<dec << "Bad Dog in UC_Geometry::getTubeID : " << addTube <<" " << val <<endl;
        val = -1;
      }
    } else { //veto pmt id ( starts from 0)
      val = fTubeGUIIDForVeto[addBus-1][addSTAC-1][addTube];
			}
	}
  return val;
}


int UC_Geometry::getTubeAddress(int tubeID, int &addBus, int &addSTAC, int &addTube)
{
  
  //first find the row we are in:
  int row = int(tubeID/32);
  
  //position in the row:
  int pos = (tubeID - 32*row);
  
  //stac for that position
  int stac = int(pos/4);
  
  //row within stac
  int rws = (row - 4*int(row/4));
  
  addBus  = int(row/4)+1;
  addSTAC = stac + 1;
  addTube = rws*4 + (pos-4*stac);
  
  //cout << "rws: " << rws << endl;
  //cout << "pos: " << pos << endl;
  //cout << "stc: " << stac << endl;
  
  return 0;
}
